package com.yeejoin.amos.boot.module.jg.biz.edit.process.biz;

import com.alibaba.fastjson.JSONObject;
import com.yeejoin.amos.boot.module.jg.api.dto.BizRelationDataDto;
import com.yeejoin.amos.boot.module.jg.api.dto.FieldChangeMeta;
import com.yeejoin.amos.boot.module.jg.biz.context.EquipDataProcessStrategyContext;
import com.yeejoin.amos.boot.module.jg.biz.edit.core.IEventPublisher;
import com.yeejoin.amos.boot.module.jg.biz.edit.event.BaseBizDataChangeEvent;
import com.yeejoin.amos.boot.module.jg.biz.edit.process.biz.strategy.IBizDataChangeHandleStrategy;
import com.yeejoin.amos.boot.module.jg.biz.edit.process.equip.strategy.IEquipChangeDataProcessStrategy;
import org.typroject.tyboot.core.foundation.context.RequestContext;

import java.util.List;
import java.util.Map;

public abstract class DefaultBizDataChangeHandler<U extends BaseBizDataChangeEvent> implements IBizDataChangeHandleStrategy {

    private final IEventPublisher<BaseBizDataChangeEvent> eventPublisher;

    protected DefaultBizDataChangeHandler(IEventPublisher<BaseBizDataChangeEvent> eventPublisher) {
        this.eventPublisher = eventPublisher;
    }


    @Override
    public final void doSave(String applyNo, ModelType model, Map<String, Object> changeData) {
        if (beforeCheck(applyNo, model, changeData)) {
            JSONObject oData = new JSONObject(changeData);
            // 删除原始提交的变更说明及附件放置后续存放到json
            changeData.remove("changeReason");
            changeData.remove("changeAttachment");
            IEquipChangeDataProcessStrategy dataProcessor = EquipDataProcessStrategyContext.getStrategy(model);
            List<FieldChangeMeta> allChangeColumns = dataProcessor.handle(changeData, applyNo);
            List<FieldChangeMeta> bizEditColumns = updBizData(applyNo, model, changeData);
            allChangeColumns.addAll(bizEditColumns);
            if(bizIsFinished(applyNo, model, changeData)){
                List<FieldChangeMeta> equipColumns = updEqDataIfBizFin(applyNo, model, changeData);
                allChangeColumns.addAll(equipColumns);
            }
            // 发送数据变更消息
            publish2OtherBiz(allChangeColumns, applyNo, oData);
        }
    }

    private void publish2OtherBiz(List<FieldChangeMeta> allChangeColumns, String applyNo, JSONObject changeData) {
        if(!allChangeColumns.isEmpty()){
            BizRelationDataDto bizRelationDataDto = new BizRelationDataDto();
            bizRelationDataDto.setBizId(applyNo);
            bizRelationDataDto.setBizType(canHandleBizType());
            bizRelationDataDto.setRecords(getEqs(applyNo));
            bizRelationDataDto.setChangeReason(changeData.getString("changeReason"));
            bizRelationDataDto.setChangeAttachment(JSONObject.toJSONString(changeData.get("changeAttachment")));
            bizRelationDataDto.setProjectContraptionIds(getProjectContraptionIds(applyNo));
            eventPublisher.publish(new BaseBizDataChangeEvent(this, bizRelationDataDto, allChangeColumns, RequestContext.cloneRequestContext()));
        }
    }
}
