package com.yeejoin.amos.boot.module.jg.biz.edit.process.biz.useRegister;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.yeejoin.amos.boot.biz.common.entity.BaseEntity;
import com.yeejoin.amos.boot.module.jg.api.dto.FieldChangeMeta;
import com.yeejoin.amos.boot.module.jg.api.dto.UseRegisterChangeDataDto;
import com.yeejoin.amos.boot.module.jg.api.entity.JgUseRegistration;
import com.yeejoin.amos.boot.module.jg.api.entity.JgUseRegistrationEq;
import com.yeejoin.amos.boot.module.jg.biz.edit.core.RouterEventPublisher;
import com.yeejoin.amos.boot.module.jg.biz.edit.event.BaseBizDataChangeEvent;
import com.yeejoin.amos.boot.module.jg.biz.edit.process.biz.DefaultBizDataChangeHandler;
import com.yeejoin.amos.boot.module.jg.biz.edit.process.equip.CommonEquipDataProcessService;
import com.yeejoin.amos.boot.module.jg.biz.service.impl.JgUseRegistrationServiceImpl;
import com.yeejoin.amos.boot.module.ymt.api.enums.FlowStatusEnum;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class UseRegisterDataChangeHandler extends DefaultBizDataChangeHandler<BaseBizDataChangeEvent> {


    private final UseRegisterUpdateService useRegisterUpdateService;


    protected UseRegisterDataChangeHandler(RouterEventPublisher routerEventPublisher, UseRegisterUpdateService useRegisterUpdateService) {
        super(routerEventPublisher);
        this.useRegisterUpdateService = useRegisterUpdateService;
    }


    @Override
    public String canHandleBizType() {
        return "useRegistration";
    }


    /**
     * 保存
     *
     * @param applyNo    单据编号
     * @param model      single, batch
     * @param changeData 前端不支持分组-目前为平铺数据结构
     */
    @Override
    public List<FieldChangeMeta> updBizData(String applyNo, ModelType model, Map<String, Object> changeData) {
        List<FieldChangeMeta> bizEditColumns = new ArrayList<>();
        JgUseRegistration useRegistration = useRegisterUpdateService.getJgUseRegistrationService().getOne(new LambdaQueryWrapper<JgUseRegistration>().eq(JgUseRegistration::getApplyNo, applyNo));
        // 1.构建单据字段级变化流水
        buildBizFieldChangeLogs(changeData, useRegistration, bizEditColumns);
        // 2.更新单据
        this.updateBizInfo(changeData);
        // 3.更新历史的JSON的数据
        this.updateHistoryJson(applyNo);
        return bizEditColumns;
    }

    private void updateBizInfo(Map<String, Object> changeData) {
        useRegisterUpdateService.updateBizInfo(changeData);
    }

    private void buildBizFieldChangeLogs(Map<String, Object> changeData, JgUseRegistration useRegistration, List<FieldChangeMeta> bizEditColumns) {
        UseRegisterChangeDataDto newChangeData = CommonEquipDataProcessService.castMap2Bean(changeData, UseRegisterChangeDataDto.class);
        newChangeData.setApplyNo(useRegistration.getApplyNo());
        Map<String, Object> oldData = useRegisterUpdateService.getJgUseRegistrationService().getDetail(null, useRegistration.getSequenceNbr(), null);
        UseRegisterChangeDataDto oldChangeData = CommonEquipDataProcessService.castMap2Bean(oldData, UseRegisterChangeDataDto.class);
        List<FieldChangeMeta> oldFields = CommonEquipDataProcessService.convertBeanField2Column2(newChangeData, useRegistration.getApplyNo());
        List<FieldChangeMeta> newFields = CommonEquipDataProcessService.convertBeanField2Column2(oldChangeData, useRegistration.getApplyNo());
        bizEditColumns.addAll(CommonEquipDataProcessService.mergeChangeFields(newFields, oldFields));
    }

    @Override
    public Boolean beforeCheck(String applyNo, ModelType model, Map<String, Object> changeData) {
        return useRegisterUpdateService.beforeCheck(changeData, applyNo);
    }

    @Override
    public Set<String> getEqs(String applyNo) {
        JgUseRegistration useRegistration = useRegisterUpdateService.getJgUseRegistrationService().getOne(new LambdaQueryWrapper<JgUseRegistration>().select(BaseEntity::getSequenceNbr).eq(JgUseRegistration::getApplyNo, applyNo));
        List<JgUseRegistrationEq> eqs = useRegisterUpdateService.getUseRegistrationEqService().list(new LambdaQueryWrapper<JgUseRegistrationEq>().select(JgUseRegistrationEq::getEquId).eq(JgUseRegistrationEq::getEquipTransferId, useRegistration.getSequenceNbr()));
        return eqs.stream().map(JgUseRegistrationEq::getEquId).collect(Collectors.toSet());
    }

    @Override
    public List<FieldChangeMeta> updEqDataIfBizFin(String applyNo, ModelType model, Map<String, Object> changeData) {
        // 更新设备/装置的使用信息
        return useRegisterUpdateService.updEqData(applyNo, new JSONObject(changeData));
    }

    @Override
    public Boolean bizIsFinished(String applyNo, ModelType model, Map<String, Object> changeData) {
        JgUseRegistration useRegistration = useRegisterUpdateService.getJgUseRegistrationService().getOne(new LambdaQueryWrapper<JgUseRegistration>().eq(JgUseRegistration::getApplyNo, applyNo));
        return useRegistration.getStatus().equals(FlowStatusEnum.TO_BE_FINISHED.getName());
    }

    @Override
    public Set<String> getProjectContraptionIds(String applyNo) {
        JgUseRegistration useRegistration = useRegisterUpdateService.getJgUseRegistrationService().getOne(new LambdaQueryWrapper<JgUseRegistration>().select(BaseEntity::getSequenceNbr, JgUseRegistration::getApplyNo, JgUseRegistration::getProjectContraptionId).eq(JgUseRegistration::getApplyNo, applyNo));
        Set<String> projectContraptionIds = new HashSet<>();
        projectContraptionIds.add(useRegistration.getProjectContraptionId());
        projectContraptionIds.remove(null);
        projectContraptionIds.remove("");
        return projectContraptionIds;
    }

    private void updateHistoryJson(String applyNo) {
        useRegisterUpdateService.updateHisData(applyNo);
    }


    @Override
    public Map<String, ?> getDetail(String applyNo, String bizId) {
        Map<String, Object> detail = new HashMap<>();
        JgUseRegistration useRegistration = useRegisterUpdateService.getJgUseRegistrationService().getOne(new LambdaQueryWrapper<JgUseRegistration>().eq(JgUseRegistration::getApplyNo, applyNo));
        JSONObject lastDetailDataForEdit = useRegisterUpdateService.getLastDetailDataForEdit(useRegistration);
        BeanUtil.copyProperties(lastDetailDataForEdit, detail);
        BeanUtil.copyProperties(useRegistration, detail);
        detail.put("equipId", lastDetailDataForEdit.getString("equipId"));
        // 转化 附件 字段
        useRegisterUpdateService.convertStringToJsonObject(detail, JgUseRegistrationServiceImpl.jsonFields);
        return detail;
    }

}
