package com.yeejoin.amos.boot.module.jg.biz.edit.process.biz.useRegister;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.map.MapBuilder;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yeejoin.amos.boot.biz.common.bo.CompanyBo;
import com.yeejoin.amos.boot.biz.common.bo.ReginParams;
import com.yeejoin.amos.boot.biz.common.utils.DateUtils;
import com.yeejoin.amos.boot.biz.common.utils.RedisKey;
import com.yeejoin.amos.boot.biz.common.utils.RedisUtils;
import com.yeejoin.amos.boot.module.jg.api.dto.FieldChangeMeta;
import com.yeejoin.amos.boot.module.jg.api.entity.JgRegistrationHistory;
import com.yeejoin.amos.boot.module.jg.api.entity.JgUseRegistration;
import com.yeejoin.amos.boot.module.jg.api.entity.JgUseRegistrationEq;
import com.yeejoin.amos.boot.module.jg.api.enums.CompanyTypeEnum;
import com.yeejoin.amos.boot.module.jg.api.enums.CylinderTypeEnum;
import com.yeejoin.amos.boot.module.jg.api.mapper.CommonMapper;
import com.yeejoin.amos.boot.module.jg.biz.edit.process.equip.CommonEquipDataProcessService;
import com.yeejoin.amos.boot.module.jg.biz.service.impl.IdxBizJgProjectContraptionServiceImpl;
import com.yeejoin.amos.boot.module.jg.biz.service.impl.JgUseRegistrationEqServiceImpl;
import com.yeejoin.amos.boot.module.jg.biz.service.impl.JgUseRegistrationServiceImpl;
import com.yeejoin.amos.boot.module.ymt.api.entity.*;
import com.yeejoin.amos.boot.module.ymt.api.enums.EquipmentClassifityEnum;
import com.yeejoin.amos.boot.module.ymt.api.enums.InformationManageTypeEnum;
import com.yeejoin.amos.boot.module.ymt.api.mapper.IdxBizJgRegisterInfoMapper;
import com.yeejoin.amos.feign.privilege.Privilege;
import com.yeejoin.amos.feign.privilege.model.CompanyModel;
import com.yeejoin.amos.feign.systemctl.Systemctl;
import com.yeejoin.amos.feign.systemctl.model.DictionarieValueModel;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.typroject.tyboot.core.foundation.context.RequestContext;
import org.typroject.tyboot.core.foundation.utils.ValidationUtil;
import org.typroject.tyboot.core.restful.exception.instance.BadRequest;

import javax.annotation.PostConstruct;
import java.text.ParseException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.yeejoin.amos.boot.module.jg.biz.service.impl.JgUseRegistrationServiceImpl.carNumber;
import static java.util.stream.Collectors.toList;

@Service
@RequiredArgsConstructor
@Getter
@Slf4j
public class UseRegisterUpdateService {

    private final IdxBizJgRegisterInfoMapper idxBizJgRegisterInfoMapper;

    private final IdxBizJgProjectContraptionServiceImpl idxBizJgProjectContraptionService;

    private final RedisUtils redisUtils;

    private final JgUseRegistrationServiceImpl jgUseRegistrationService;

    private final Map<String, Function<JSONObject, Boolean>> bizHandlers = new ConcurrentHashMap<>();

    private final CommonEquipDataProcessService commonEquipDataProcessService;

    private final CommonMapper commonMapper;

    private final JgUseRegistrationEqServiceImpl useRegistrationEqService;

    @PostConstruct
    public void init() {
        // 新增台套登记
        bizHandlers.put("0-set", this::handleUseRegistrationEdit);
        bizHandlers.put("-set", this::handleUseRegistrationEdit);
        // 新增单位登记
        bizHandlers.put("0-unit", this::handleUnitUseRegistrationEdit);
        // 历史有证台套登记
        bizHandlers.put("1-set", this::handleHistoryEquipEdit);
        // 历史无证台套登记
        bizHandlers.put("2-set", this::handleUseRegistrationEdit);
        // 历史无证单位登记
        bizHandlers.put("2-unit", this::handleUnitUseRegistrationEdit);
    }

    public void updateHisData(String applyNo) {
        JgUseRegistration jgUseRegistration = jgUseRegistrationService.getOne(new LambdaQueryWrapper<JgUseRegistration>().eq(JgUseRegistration::getApplyNo, applyNo));
        // 构建更新json数据
        JSONObject hisData = this.buildLastDetailData(jgUseRegistration);
        // 更新数据
        jgUseRegistrationService.updateHistory(hisData, null, jgUseRegistration.getSequenceNbr().toString(), jgUseRegistration.getSupervisoryCode());
    }

    public JSONObject buildLastDetailData(JgUseRegistration jgUseRegistration) {
        JgRegistrationHistory jgRegistrationHistory = getJgRegistrationHistory(jgUseRegistration);
        JSONObject jsonObject = JSONObject.parseObject(jgRegistrationHistory.getChangeData());
        jsonObject.put("receiveOrgCode", jgUseRegistration.getReceiveCompanyCode() + "_" + jgUseRegistration.getReceiveOrgName());
        jsonObject.put("status", jgUseRegistration.getStatus());
        jsonObject.put("applyNo", jgUseRegistration.getApplyNo());
        jsonObject.remove("submit");
        jsonObject.remove("instanceId");
        if (!ObjectUtils.isEmpty(jsonObject.get("factoryUseSiteStreet"))) {
            jsonObject.put("factoryUseSiteStreet", String.valueOf(Long.valueOf(jsonObject.get("factoryUseSiteStreet").toString())));
        }
        if (!"unit".equals(jgUseRegistration.getManageType())) {
            // 台套逻辑
            String record = jgUseRegistrationService.setDefaultRecord(jgUseRegistration, null);
            // 更新 设计、制造、基本信息json
            jgUseRegistrationService.fillLastEquipBaseInfo(jsonObject, record);
            // 补充"车牌号"字段
            LambdaQueryWrapper<IdxBizJgRegisterInfo> wrapper = new QueryWrapper<IdxBizJgRegisterInfo>().lambda().eq(IdxBizJgRegisterInfo::getRecord, record);
            IdxBizJgRegisterInfo idxBizJgRegisterInfo = idxBizJgRegisterInfoMapper.selectOne(wrapper);
            jsonObject.put("carNumber", idxBizJgRegisterInfo.getCarNumber());
            jsonObject.put("regType", jgUseRegistration.getRegType());
        } else {
            // 单位办理逻辑
            List<Map<String, Object>> equipmentLists = (List<Map<String, Object>>) jsonObject.get("equipmentLists");
            List<String> records = equipmentLists.stream().map(v -> (String) v.get("record")).collect(toList());
            jsonObject.remove("equipmentLists");
            if (CylinderTypeEnum.CYLINDER.getCode().equals(jsonObject.get("EQU_CATEGORY_CODE"))) {
                List<DictionarieValueModel> fillingMedium = Systemctl.dictionarieClient.dictValues("FILLING_MEDIUM").getResult();
                Map<String, Object> fillingMediumMap = fillingMedium.stream().collect(Collectors.toMap(DictionarieValueModel::getDictDataKey, DictionarieValueModel::getDictDataValue));
                List<Map<String, Object>> result = jgUseRegistrationService.getBaseMapper().queryForUnitVesselEquipment(records);
                result.forEach(i -> {
                    i.put("chargingMedium", fillingMediumMap.get(i.get("chargingMedium")));
                    i.put("productPhoto", JSONArray.parseArray((String) i.get("productPhoto")));
                    i.put("cylinderStampAttachment", JSONArray.parseArray((String) i.get("cylinderStampAttachment")));
                    i.put("informationSituation", InformationManageTypeEnum.getName((String) i.get("informationSituation")));
                });
                jsonObject.put("equipmentLists", result);
            } else {
                // 管道逻辑 更新装置、管道有关数据
                IdxBizJgProjectContraption projectContraption = idxBizJgProjectContraptionService.getById(jgUseRegistration.getProjectContraptionId());
                jsonObject.put("pipelineLength", projectContraption.getPipelineLength());
                jsonObject.put("projectContraption", projectContraption.getProjectContraption());
                jsonObject.put("projectContraptionNo", projectContraption.getProjectContraptionNo());
                List<Map<String, Object>> equList = jgUseRegistrationService.getBaseMapper().queryForUnitPipelineEquipment(records);
                // 检验报告数据格式化 转json
                equList.stream().filter(e -> e.get("inspectReport") != null).forEach(item -> item.put("inspectReport", JSON.parse(item.get("inspectReport").toString())));
                jsonObject.put("equipmentLists", equList);
            }
        }
        jsonObject.put("useRegistrationCode", jgUseRegistration.getUseRegistrationCode());
        // 转化 附件 字段
        this.convertStringToJsonObject(jsonObject, JgUseRegistrationServiceImpl.jsonFields);
        return jsonObject;
    }

    public JSONObject getLastDetailDataForEdit(JgUseRegistration jgUseRegistration) {
        JgRegistrationHistory jgRegistrationHistory = getJgRegistrationHistory(jgUseRegistration);
        JSONObject jsonObject = JSONObject.parseObject(jgRegistrationHistory.getChangeData());
        jsonObject.put("receiveOrgCode", jgUseRegistration.getReceiveCompanyCode() + "_" + jgUseRegistration.getReceiveOrgName());
        jsonObject.put("status", jgUseRegistration.getStatus());
        jsonObject.put("applyNo", jgUseRegistration.getApplyNo());
        jsonObject.remove("submit");
        jsonObject.remove("instanceId");
        if (!"unit".equals(jgUseRegistration.getManageType())) {
            // 台套逻辑
            String record = jgUseRegistrationService.setDefaultRecord(jgUseRegistration, null);
            // 更新 设计、制造、基本信息json
            jgUseRegistrationService.fillLastEquipBaseInfo(jsonObject, record);
            // 补充"车牌号"字段
            LambdaQueryWrapper<IdxBizJgRegisterInfo> wrapper = new QueryWrapper<IdxBizJgRegisterInfo>().lambda().eq(IdxBizJgRegisterInfo::getRecord, record);
            IdxBizJgRegisterInfo idxBizJgRegisterInfo = idxBizJgRegisterInfoMapper.selectOne(wrapper);
            jsonObject.put("carNumber", idxBizJgRegisterInfo.getCarNumber());
            jsonObject.put("regType", jgUseRegistration.getRegType());
        } else {
            // 单位办理逻辑
            List<Map<String, Object>> equipmentLists = (List<Map<String, Object>>) jsonObject.get("equipmentLists");
            List<String> records = equipmentLists.stream().map(v -> (String) v.get("record")).collect(toList());
            jsonObject.remove("equipmentLists");
            if (CylinderTypeEnum.CYLINDER.getCode().equals(jsonObject.get("EQU_CATEGORY_CODE"))) {
                List<DictionarieValueModel> fillingMedium = Systemctl.dictionarieClient.dictValues("FILLING_MEDIUM").getResult();
                Map<String, Object> fillingMediumMap = fillingMedium.stream().collect(Collectors.toMap(DictionarieValueModel::getDictDataKey, DictionarieValueModel::getDictDataValue));
                List<Map<String, Object>> result = jgUseRegistrationService.getBaseMapper().queryForUnitVesselEquipment(records);
                result.forEach(i -> {
                    i.put("chargingMedium", fillingMediumMap.get(i.get("chargingMedium")));
                    i.put("productPhoto", JSONArray.parseArray((String) i.get("productPhoto")));
                    i.put("cylinderStampAttachment", JSONArray.parseArray((String) i.get("cylinderStampAttachment")));
                    i.put("informationSituation", InformationManageTypeEnum.getName((String) i.get("informationSituation")));
                });
                jsonObject.put("tableData", result);
            } else {
                // 管道逻辑 更新装置、管道有关数据
                IdxBizJgProjectContraption projectContraption = idxBizJgProjectContraptionService.getById(jgUseRegistration.getProjectContraptionId());
                jsonObject.put("pipelineLength", projectContraption.getPipelineLength());
                jsonObject.put("projectContraption", projectContraption.getProjectContraption());
                jsonObject.put("projectContraptionNo", projectContraption.getProjectContraptionNo());
                List<Map<String, Object>> equList = jgUseRegistrationService.getBaseMapper().queryForUnitPipelineEquipment(records);
                // 检验报告数据格式化 转json
                equList.stream().filter(e -> e.get("inspectReport") != null).forEach(item -> item.put("inspectReport", JSON.parse(item.get("inspectReport").toString())));
                jsonObject.put("tableData", equList);
            }
        }
        jsonObject.put("useRegistrationCode", jgUseRegistration.getUseRegistrationCode());
        return jsonObject;
    }

    public JgRegistrationHistory getJgRegistrationHistory(JgUseRegistration jgUseRegistration) {
        LambdaQueryWrapper<JgRegistrationHistory> lambda = new QueryWrapper<JgRegistrationHistory>().lambda();
        lambda.eq(JgRegistrationHistory::getCurrentDocumentId, jgUseRegistration.getSequenceNbr());
        lambda.eq(JgRegistrationHistory::getIsDelete, false);
        return jgUseRegistrationService.getJgRegistrationHistoryService().getBaseMapper().selectOne(lambda);
    }

    public void convertStringToJsonObject(Object obj, String[] jsonFields) {
        if (obj instanceof JSONObject) {
            JSONObject jsonObject = (JSONObject) obj;
            for (String field : jsonFields) {
                if (jsonObject.get(field) != null && jsonObject.get(field) instanceof String) {
                    jsonObject.put(field, JSON.parse(JSONObject.toJSONString(jsonObject.get(field))));
                }
            }
        } else if (obj instanceof Map) {
            Map<String, Object> map = (Map<String, Object>) obj;
            for (String field : jsonFields) {
                if (map.containsKey(field) && map.get(field) instanceof String) {
                    map.put(field, JSON.parse((String) map.get(field)));
                }
            }
        } else {
            // 处理其他类型
            System.out.println("Unsupported Object Type");
        }
    }

    public Boolean handleUnitUseRegistrationEdit(JSONObject map) {
        JgUseRegistration useRegistration = JSON.parseObject(JSON.toJSONString(map), JgUseRegistration.class);
        List<Map<String, Object>> equipmentLists = (List<Map<String, Object>>) map.get("equipmentLists");
        useRegistration.setRegDate(new Date());
        useRegistration.setCreateDate(new Date());
        useRegistration.setCylinderCategory(map.getString("cylinderCategory"));//气瓶分类
        // 接收单位信息
        Optional.ofNullable(useRegistration.getReceiveOrgCode())
                .filter(code -> code.contains("_"))
                .map(code -> code.split("_"))
                .ifPresent(splitReceiveOrgCode -> {
                    CompanyModel result = Privilege.companyClient.queryByCompanyCode(splitReceiveOrgCode[0]).getResult();
                    useRegistration.setReceiveOrgCode(splitReceiveOrgCode[0]);
                    useRegistration.setReceiveCompanyOrgCode(splitReceiveOrgCode[0]);
                    useRegistration.setReceiveCompanyCode(result.getCompanyCode());
                    useRegistration.setReceiveOrgName(splitReceiveOrgCode[1]);
                });

        // 安全管理员
        Optional.ofNullable(map.getString("safetyManager"))
                .filter(manager -> manager.contains("_"))
                .map(manager -> manager.split("_"))
                .ifPresent(data -> {
                    map.put("safetyManagerId", data[0]);
                    map.put("safetyManagerName", data[1]);
                });

        // 其他附件
        if (!ObjectUtils.isEmpty(map.get("otherAccessories"))) {
            useRegistration.setOtherAccessories(JSONObject.toJSONString(map.get("otherAccessories")));
        }
        if (map.containsKey("orgBranchCode")) {
            // 监察处置机构代码
            String[] splitOrgBranchCode = String.valueOf(map.getString("orgBranchCode")).split("_");
            useRegistration.setSupervisionOrgCode(splitOrgBranchCode[0]);
        }

        // 使用地点
        Map<String, List<LinkedHashMap>> locationMap = MapBuilder.<String, List<LinkedHashMap>>create()
                .put("city", (List<LinkedHashMap>) redisUtils.get("CITY"))
                .put("county", (List<LinkedHashMap>) redisUtils.get("REGION"))
                .put("factoryUseSiteStreet", (List<LinkedHashMap>) redisUtils.get("STREET"))
                .build();
        String fullAddress = locationMap.entrySet().stream()
                .map(entry -> {
                    String mapKey = String.valueOf(map.get(entry.getKey()));
                    List<LinkedHashMap> locations = entry.getValue();
                    return (!ObjectUtils.isEmpty(mapKey) && !ObjectUtils.isEmpty(locations)) ?
                            locations.stream()
                                    .filter(item -> mapKey.equals(String.valueOf(item.get("regionCode"))))
                                    .map(item -> String.valueOf(item.get("regionName")))
                                    .findFirst()
                                    .orElse("") :
                            "";
                })
                .collect(Collectors.joining());

        // 具体地址
        useRegistration.setUseAddress("陕西省" + fullAddress + map.get("address"));
        useRegistration.setOriginProjectContraptionIds((String) map.get("originProjectContraptionIds"));
        // 使用登记表
        Optional.ofNullable(map.get("useRegistrationFormFile"))
                .map(JSONObject::toJSONString)
                .ifPresent(useRegistration::setUseRegistrationFormFile);
        Map<String, Object> useRegistrationFormUrlMap = jgUseRegistrationService.getCommonService().getRegistrationFormUrl("unit", map);
        Optional.ofNullable(useRegistrationFormUrlMap)
                .ifPresent(item -> useRegistration.setUseRegistrationFormUrl(String.valueOf(item.get("useRegistrationFormUrl"))));

        jgUseRegistrationService.updateById(useRegistration);
        // 删除设备关联表记录
        jgUseRegistrationService.getJgUseRegistrationEqService().getBaseMapper().delete(new QueryWrapper<JgUseRegistrationEq>().lambda().eq(JgUseRegistrationEq::getEquipTransferId, useRegistration.getSequenceNbr()));
        // 更新关联气瓶信息
        List<JgUseRegistrationEq> equipList = equipmentLists.stream()
                .map(x -> new JgUseRegistrationEq()
                        .setEquId(String.valueOf(x.get("record")))
                        .setOriginProjectContraptionId(String.valueOf(x.get("projectContraptionId")))
                        .setEquipTransferId(String.valueOf(useRegistration.getSequenceNbr())))
                .collect(Collectors.toList());
        // 保存关联设备信息
        jgUseRegistrationService.getJgUseRegistrationEqService().saveBatch(equipList);
        return null;
    }

    public Boolean beforeCheck(Map<String, Object> map, String applyNo) {
        JgUseRegistration useRegistration = JSON.parseObject(JSON.toJSONString(map), JgUseRegistration.class);
        // 新增登记单位办理校验
        if (useRegistration.getManageType().equals("unit")) {
            String equListCode = String.valueOf(map.get("EQU_LIST_CODE"));
            List<Map<String, Object>> equipmentLists = (List<Map<String, Object>>) map.get("equipmentLists");
            if (CollectionUtils.isEmpty(equipmentLists)) {
                throw new BadRequest("请选择设备信息!");
            }
            if (CylinderTypeEnum.CYLINDER.getCode().equals(map.get("EQU_CATEGORY_CODE"))) {
                boolean hasExistingCertificate = jgUseRegistrationService.getBaseMapper().selectList(
                        new LambdaQueryWrapper<JgUseRegistration>()
                                .eq(JgUseRegistration::getUseUnitCreditCode, useRegistration.getUseUnitCreditCode())
                                .eq(JgUseRegistration::getIsDelete, false)
                                .eq(JgUseRegistration::getStatus, "已完成")
                ).stream().anyMatch(v -> !ObjectUtils.isEmpty(v.getUseRegistrationCode()) && v.getUseRegistrationCode().contains("瓶30"));
                if (hasExistingCertificate && map.get("isAddEquip").equals("0")) {
                    throw new BadRequest("该企业已存在气瓶证书，请选择增补功能进行登记！");
                }
            }
            if (EquipmentClassifityEnum.YLGD.getCode().equals(equListCode)) {
                equipmentLists.forEach(equip -> {
                    if ((equip.get("inspectReportNo") == null || equip.get("inspectReportNo").toString().isEmpty()) ||
                            (equip.get("inspectOrgCode") == null || equip.get("inspectOrgCode").toString().isEmpty())) {
                        throw new BadRequest("检验信息不能为空，请完善检验信息后进行登记！");
                    }
                });
            }
        }
        // 历史登记台套（无按照单位办理业务）
        if (useRegistration.getRegType().equals("1")) {
            // 使用登记证编号
            String useRegistrationCode = String.valueOf(map.get("useOrgCode")).trim();
            String equipId = String.valueOf(map.get("equipId"));
            //校验使用登记证编号的唯一性
            Boolean used = jgUseRegistrationService.getCommonService().useRegistrationCertificateAccountUnique(useRegistrationCode, equipId);
            if (used) {
                throw new BadRequest("使用登记证编号已存在！");
            }
            //使用登记证编号判断是否使用未来系统生成编号
            jgUseRegistrationService.getIdxBizJgRegisterInfoService().checkUseRegistrationCode(useRegistrationCode, "set");

            // 场车设备校验车牌号的唯一性
            if (EquipmentClassifityEnum.CC.getCode().equals(map.get("EQU_LIST_CODE"))) {
                String carNum = String.valueOf(map.get(carNumber));
                if (!"无".equals(carNum)) {
                    Integer count = idxBizJgRegisterInfoMapper.checkCarNumberUniquenessWithHisCC(carNumber, equipId);
                    if (count > 0) {
                        throw new BadRequest("车牌号已存在，请重新输入！");
                    }
                }
            }
        }
        return true;
    }

    public Boolean handleUseRegistrationEdit(JSONObject map) {
        // 使用登记信息
        JgUseRegistration jgUseRegistration = new JgUseRegistration();
        jgUseRegistration.setRecDate(new Date());
        jgUseRegistration.setOtherAccessories(JSONObject.toJSONString(map.get("otherAccessories")));
        if (map.containsKey("orgBranchCode")) {
            // 监察处置机构代码
            String[] splitOrgBranchCode = String.valueOf(map.getString("orgBranchCode")).split("_");
            jgUseRegistration.setSupervisionOrgCode(splitOrgBranchCode[0]);
        }
        // 安全管理员
        if (map.containsKey("safetyManager")) {
            String[] data = String.valueOf(map.getString("safetyManager")).split("_");
            map.put("safetyManagerId", data[0]);
            map.put("safetyManagerName", data[1]);
        }
        jgUseRegistration.setInspectUnitCreditCode(map.getString("inspectUnitCreditCode"));
        jgUseRegistration.setInspectUnitName(map.getString("inspectOrgName"));
        // 是否西咸
        jgUseRegistration.setIsXixian(map.getString("isXixian"));
        // TODO 待优化提取到公共类-使用一份内存
        setUseAddress(map, jgUseRegistration);
        jgUseRegistration.setUseRegistrationFormFile(JSONObject.toJSONString(map.get("useRegistrationFormFile")));
        Map<String, Object> useRegistrationFormUrlMap = jgUseRegistrationService.getCommonService().getRegistrationFormUrl("set", map);
        Optional.ofNullable(useRegistrationFormUrlMap)
                .ifPresent(item -> jgUseRegistration.setUseRegistrationFormUrl(String.valueOf(item.get("useRegistrationFormUrl"))));

        jgUseRegistration.setSequenceNbr(map.getLong("sequenceNbr"));
        jgUseRegistrationService.getBaseMapper().updateById(jgUseRegistration);
        // 更新设备关联表
        LambdaQueryWrapper<JgUseRegistrationEq> lambda = new QueryWrapper<JgUseRegistrationEq>().lambda();
        lambda.eq(JgUseRegistrationEq::getEquipTransferId, jgUseRegistration.getSequenceNbr());
        JgUseRegistrationEq jgUseRegistrationEq = new JgUseRegistrationEq();
        jgUseRegistrationEq.setEquId(map.get("equipId").toString());
        jgUseRegistrationService.getJgUseRegistrationEqService().update(jgUseRegistrationEq, lambda);
        // TODO 更新历史数据-主要是业务字段（如管理员等没在主表存储）
        jgUseRegistrationService.updateHistory(map, map.get("equipId").toString(), String.valueOf(jgUseRegistration.getSequenceNbr()), jgUseRegistration.getSupervisoryCode());
        // 历史有证设备登记生成的 编辑时更新使用登记证管理表
        jgUseRegistrationService.updateRegistrationManage(map.getLong("sequenceNbr"));
        return Boolean.TRUE;
    }

    private void setUseAddress(JSONObject map, JgUseRegistration jgUseRegistration) {
        // 市
        List<LinkedHashMap> city = (List<LinkedHashMap>) redisUtils.get("CITY");
        // 区
        List<LinkedHashMap> region = (List<LinkedHashMap>) redisUtils.get("REGION");
        // 街道
        List<LinkedHashMap> street = (List<LinkedHashMap>) redisUtils.get("STREET");
        jgUseRegistration.setUseAddress("陕西省");
        // 城市
        if (!ObjectUtils.isEmpty(map.get("city")) && !ObjectUtils.isEmpty(city)) {
            city.forEach(item -> {
                if (String.valueOf(item.get("regionCode")).equals(String.valueOf(map.get("city")))) {
                    jgUseRegistration.setUseAddress(jgUseRegistration.getUseAddress() + item.get("regionName"));
                }
            });
        }
        // 区县
        if (!ObjectUtils.isEmpty(map.get("county")) && !ObjectUtils.isEmpty(region)) {
            region.forEach(item -> {
                if (String.valueOf(item.get("regionCode")).equals(String.valueOf(map.get("county")))) {
                    jgUseRegistration.setUseAddress(jgUseRegistration.getUseAddress() + item.get("regionName"));
                }
            });
        }
        // 街道
        if (!ObjectUtils.isEmpty(map.get("factoryUseSiteStreet")) && !ObjectUtils.isEmpty(street)) {
            street.forEach(item -> {
                if (String.valueOf(item.get("regionCode")).equals(String.valueOf(map.get("factoryUseSiteStreet")))) {
                    jgUseRegistration.setUseAddress(jgUseRegistration.getUseAddress() + item.get("regionName"));
                }
            });
        }
        jgUseRegistration.setUseAddress(jgUseRegistration.getUseAddress() + map.getString("address"));
    }

    /**
     * 历史台套登记
     *
     * @param map 提交参数
     */
    public Boolean handleHistoryEquipEdit(JSONObject map) {
        // 使用登记证编号
        ReginParams reginParams = JSONObject.parseObject(redisUtils.get(RedisKey.buildReginKey(RequestContext.getExeUserId(), RequestContext.getToken())).toString(), ReginParams.class);
        CompanyBo company = reginParams.getCompany();
        // 使用登记信息
        JgUseRegistration jgUseRegistration = new JgUseRegistration();
        jgUseRegistration.setRegDate(new Date());
        if (!ObjectUtils.isEmpty(map.get("otherAccessories"))) {
            jgUseRegistration.setOtherAccessories(JSONObject.toJSONString(map.get("otherAccessories")));
        }
        if (map.containsKey("receiveOrgCode")) {
            // 接收单位信息
            String[] splitMaintenanceUnitCode = String.valueOf(map.getString("receiveOrgCode")).split("_");
            jgUseRegistration.setReceiveCompanyCode(splitMaintenanceUnitCode[0]);
            jgUseRegistration.setReceiveOrgName(splitMaintenanceUnitCode[1]);
            jgUseRegistration.setReceiveCompanyOrgCode(jgUseRegistrationService.getCommonService().getOneCompany(jgUseRegistration.getReceiveCompanyCode()).getOrgCode());
        }
        if (map.containsKey("orgBranchCode")) {
            // 监察处置机构代码
            String[] splitOrgBranchCode = String.valueOf(map.getString("orgBranchCode")).split("_");
            jgUseRegistration.setSupervisionOrgCode(splitOrgBranchCode[0]);
        }
        // 安全管理员
        if (map.containsKey("safetyManager")) {
            String[] data = String.valueOf(map.getString("safetyManager")).split("_");
            map.put("safetyManagerId", data[0]);
            map.put("safetyManagerName", data[1]);
        }
        // 使用单位提交
        jgUseRegistration.setUseUnitName(CompanyTypeEnum.INDIVIDUAL.getName().equals(company.getCompanyType()) ?
                company.getCompanyName().split("_")[1] : company.getCompanyName());
        jgUseRegistration.setUseUnitCreditCode(CompanyTypeEnum.INDIVIDUAL.getName().equals(company.getCompanyType()) ?
                company.getCompanyCode().split("_")[1] : company.getCompanyCode());
        jgUseRegistration.setCreateUserId(reginParams.getUserModel().getUserId());
        jgUseRegistration.setCreateUserName(reginParams.getUserModel().getUserName());
        if (!ObjectUtils.isEmpty(map.get("inspectUnitCreditCode"))) {
            jgUseRegistration.setInspectUnitCreditCode(map.get("inspectUnitCreditCode").toString());
        }
        if (!ObjectUtils.isEmpty(map.get("inspectOrgName"))) {
            jgUseRegistration.setInspectUnitName(map.get("inspectOrgName").toString());
        }
        // 是否西咸
        if (!ObjectUtils.isEmpty(map.get("isXixian"))) {
            jgUseRegistration.setIsXixian(String.valueOf(map.get("isXixian")));
        }
        // 使用地点
        String useCityName = "";
        String useCountyName = "";
        String useStreetName = "";
        // 市
        List<LinkedHashMap> city = (List<LinkedHashMap>) redisUtils.get("CITY");
        // 区
        List<LinkedHashMap> region = (List<LinkedHashMap>) redisUtils.get("REGION");
        // 街道
        List<LinkedHashMap> street = (List<LinkedHashMap>) redisUtils.get("STREET");
        jgUseRegistration.setUseAddress("陕西省");
        // 城市
        if (!ObjectUtils.isEmpty(map.get("city")) && !ObjectUtils.isEmpty(city)) {
            JSONObject finalMap = map;
            AtomicReference<String> atomicUseCity = new AtomicReference<>(useCityName);
            city.forEach(item -> {
                if (String.valueOf(item.get("regionCode")).equals(String.valueOf(finalMap.get("city")))) {
                    atomicUseCity.set("" + item.get("regionName"));
                    jgUseRegistration.setUseAddress(jgUseRegistration.getUseAddress() + item.get("regionName"));
                }
            });
            useCityName = atomicUseCity.get();
        }
        // 区县
        if (!ObjectUtils.isEmpty(map.get("county")) && !ObjectUtils.isEmpty(city)) {
            JSONObject finalMap1 = map;
            AtomicReference<String> atomicUseCounty = new AtomicReference<>(useCountyName);
            region.forEach(item -> {
                if (String.valueOf(item.get("regionCode")).equals(String.valueOf(finalMap1.get("county")))) {
                    atomicUseCounty.set("" + item.get("regionName"));
                    jgUseRegistration.setUseAddress(jgUseRegistration.getUseAddress() + item.get("regionName"));
                }
            });
            useCountyName = atomicUseCounty.get();
        }
        // 街道
        if (!ObjectUtils.isEmpty(map.get("factoryUseSiteStreet")) && !ObjectUtils.isEmpty(city)) {
            JSONObject finalMap2 = map;
            AtomicReference<String> atomicUseStreet = new AtomicReference<>(useStreetName);
            street.forEach(item -> {
                if (String.valueOf(item.get("regionCode")).equals(String.valueOf(finalMap2.get("factoryUseSiteStreet")))) {
                    atomicUseStreet.set("" + item.get("regionName"));
                    jgUseRegistration.setUseAddress(jgUseRegistration.getUseAddress() + item.get("regionName"));
                }
            });
            useStreetName = atomicUseStreet.get();
        }
        jgUseRegistration.setUseAddress(jgUseRegistration.getUseAddress() + map.get("address"));

        if (map.containsKey("edit") && "1".equals(String.valueOf(map.get("edit")))) {
            jgUseRegistration.setUseUnitCreditCode(null);
            jgUseRegistration.setSequenceNbr(Long.valueOf(String.valueOf(map.get("sequenceNbr"))));
            jgUseRegistrationService.getBaseMapper().updateById(jgUseRegistration);
            // 更新设备关联表
            LambdaQueryWrapper<JgUseRegistrationEq> lambda = new QueryWrapper<JgUseRegistrationEq>().lambda();
            lambda.eq(JgUseRegistrationEq::getEquipTransferId, jgUseRegistration.getSequenceNbr());
            JgUseRegistrationEq jgUseRegistrationEq = new JgUseRegistrationEq();
            jgUseRegistrationEq.setEquId(map.get("equipId").toString());
            jgUseRegistrationService.getJgUseRegistrationEqService().update(jgUseRegistrationEq, lambda);
        } else {
            // 业务管理设备信息保存
//                JgUseRegistrationEq jgRelationEquip = new JgUseRegistrationEq();
//                jgRelationEquip.setEquId(map.get("equipId").toString());
//                ResponseModel<List<String>> listResponseModel = tzsServiceFeignClient.applicationFormCode(ApplicationFormTypeEnum.SYDJ.getCode(), 1);
//                if (!ObjectUtils.isEmpty(listResponseModel)) {
//                    jgUseRegistration.setApplyNo(listResponseModel.getResult().get(0));
//                }
//                jgUseRegistration.setAuditPassDate(new Date());
//                jgUseRegistration.setAuditStatus(FlowStatusEnum.TO_BE_FINISHED.getName());
//                jgUseRegistration.setStatus(FlowStatusEnum.TO_BE_FINISHED.getName());
//                jgUseRegistration.setUseRegistrationCode(ValidationUtil.isEmpty(map.get("useOrgCode")) ? "" : String.valueOf(map.get("useOrgCode")));
//                jgUseRegistration.setRegType("1");//历史登记
//                this.save(jgUseRegistration);
//                jgRelationEquip.setEquipTransferId(jgUseRegistration.getSequenceNbr().toString());
//                jgRelationEquipMapper.insert(jgRelationEquip);
//
//                // 暂存历史表
//                updateHistory(map, map.get("equipId").toString(), String.valueOf(jgUseRegistration.getSequenceNbr()), jgUseRegistration.getSupervisoryCode());
//
//                //监督管理信息
//                LambdaQueryWrapper<IdxBizJgSupervisionInfo> lambdaSup = new QueryWrapper<IdxBizJgSupervisionInfo>().lambda();
//                lambdaSup.eq(IdxBizJgSupervisionInfo::getRecord, jgRelationEquip.getEquId());
//                IdxBizJgSupervisionInfo supervisionInfo = idxBizJgSupervisionInfoMapper.selectOne(lambdaSup);
//                supervisionInfo.setOrgBranchCode(String.valueOf(map.get("orgBranchCode")).split("_")[0]);
//                supervisionInfo.setOrgBranchName(String.valueOf(map.get("orgBranchCode")).split("_")[1]);
//                idxBizJgSupervisionInfoMapper.updateById(supervisionInfo);
//
//                // 注册信息
//                LambdaQueryWrapper<IdxBizJgRegisterInfo> lambdaReg = new QueryWrapper<IdxBizJgRegisterInfo>().lambda();
//                lambdaReg.eq(IdxBizJgRegisterInfo::getRecord, jgRelationEquip.getEquId());
//                IdxBizJgRegisterInfo registerInfo = idxBizJgRegisterInfoMapper.selectOne(lambdaReg);
//                registerInfo.setUseOrgCode(jgUseRegistration.getUseRegistrationCode());
//                registerInfo.setEquCodeType(String.valueOf(map.get("equCodeType")));
//                String equCode = ObjectUtils.isEmpty(map.get("equCode")) ? this.getEquCode(registerInfo, jgUseRegistration.getReceiveCompanyCode()) : String.valueOf(map.get("equCode"));
//                registerInfo.setEquCode(equCode);// 设备代码
//                registerInfo.setEquPrice(String.valueOf(map.get("equPrice")));
//                registerInfo.setProductPhoto(JSONObject.toJSONString(map.get("productPhoto")));
//                registerInfo.setOtherAccessoriesReg(JSONObject.toJSONString(map.get("otherAccessoriesReg")));
//                registerInfo.setInstallationIsComplete(map.getOrDefault("installationIsComplete", "1").toString());
//                if (EquipmentClassifityEnum.CC.getCode().equals(map.get("EQU_LIST_CODE"))) {
//                    registerInfo.setCarNumber((String) map.get(carNumber));
//                }
//                idxBizJgRegisterInfoMapper.updateById(registerInfo);
//
//                // 设计信息
//                LambdaQueryWrapper<IdxBizJgDesignInfo> lambdaDes = new QueryWrapper<IdxBizJgDesignInfo>().lambda();
//                lambdaDes.eq(IdxBizJgDesignInfo::getRecord, jgRelationEquip.getEquId());
//                IdxBizJgDesignInfo designInfo = idxBizJgDesignInfoService.getOne(lambdaDes);
//                designInfo.setDesignUnitCreditCode(MapUtil.getStr(map, "designUnitCreditCode"));
//                designInfo.setDesignUnitName(MapUtil.getStr(map, "designUnitName"));
//                designInfo.setDesignLicenseNum(MapUtil.getStr(map, "designLicenseNum"));
//                designInfo.setDesignUseDate(MapUtil.getStr(map, "designUseDate"));
//                designInfo.setDesignDate(!StringUtils.isEmpty(map.get("designDate")) ? DateUtil.parse(String.valueOf(map.get("designDate")), "yyyy-MM-dd") : null);
//                designInfo.setDrawingDo(ObjectUtils.isEmpty(map.get("drawingDo")) ? null : String.valueOf(map.get("drawingDo")));
//                designInfo.setAppraisalUnit(ObjectUtils.isEmpty(map.get("appraisalUnit")) ? null : String.valueOf(map.get("appraisalUnit")));
//                designInfo.setAppraisalDate(ObjectUtils.isEmpty(map.get("appraisalDate")) ? null : String.valueOf(map.get("appraisalDate")));
//                designInfo.setDesignDoc(JSONObject.toJSONString(map.get("designDoc")));
//                designInfo.setDesignStandard(ObjectUtils.isEmpty(map.get("designStandard")) ? null : JSONObject.toJSONString(map.get("designStandard")));
//                designInfo.setOtherAccessoriesDes(ObjectUtils.isEmpty(map.get("otherAccessoriesDes")) ? null : JSONObject.toJSONString(map.get("otherAccessoriesDes")));
//                idxBizJgDesignInfoMapper.updateById(designInfo);
//
//
//                // 制造信息
//                LambdaQueryWrapper<IdxBizJgFactoryInfo> factoryInfoWrapper = new LambdaQueryWrapper<>();
//                factoryInfoWrapper.eq(IdxBizJgFactoryInfo::getRecord, jgRelationEquip.getEquId());
//                IdxBizJgFactoryInfo idxBizJgFactoryInfo = idxBizJgFactoryInfoService.getOne(factoryInfoWrapper);
//                idxBizJgFactoryInfo.setProduceUnitCreditCode(MapUtil.getStr(map, "produceUnitCreditCode"));
//                idxBizJgFactoryInfo.setProduceUnitName(MapUtil.getStr(map, "produceUnitName"));
//                idxBizJgFactoryInfo.setFactoryNum(MapUtil.getStr(map, "factoryNum"));
//                idxBizJgFactoryInfo.setProduceLicenseNum(MapUtil.getStr(map, "produceLicenseNum"));
//                idxBizJgFactoryInfo.setProduceDate(!StringUtils.isEmpty(map.get("produceDate")) ? DateUtil.parse(String.valueOf(map.get("produceDate")), "yyyy-MM-dd") : null);
//                idxBizJgFactoryInfo.setImported(ObjectUtils.isEmpty(map.get("imported")) ? null : String.valueOf(map.get("imported")));
//                idxBizJgFactoryInfo.setProduceCountry(ObjectUtils.isEmpty(map.get("produceCountry")) ? null : String.valueOf(map.get("produceCountry")));
//                idxBizJgFactoryInfo.setOtherAccessoriesFact(ObjectUtils.isEmpty(map.get("otherAccessoriesFact")) ? null : JSONObject.toJSONString(map.get("otherAccessoriesFact")));
//                idxBizJgFactoryInfo.setProductQualityYieldProve(JSONObject.toJSONString(map.get("productQualityYieldProve")));
//                idxBizJgFactoryInfo.setInsUseMaintainExplain(ObjectUtils.isEmpty(map.get("insUseMaintainExplain")) ? null : JSONObject.toJSONString(map.get("insUseMaintainExplain")));
//                idxBizJgFactoryInfo.setFactoryStandard(ObjectUtils.isEmpty(map.get("factoryStandard")) ? null : JSONObject.toJSONString(map.get("factoryStandard")));
//                idxBizJgFactoryInfo.setFactSupervisionInspectionReport(ObjectUtils.isEmpty(map.get("factSupervisionInspectionReport")) ? null : JSONObject.toJSONString(map.get("factSupervisionInspectionReport")));
//                idxBizJgFactoryInfo.setBoilerEnergyEfficiencyCertificate(ObjectUtils.isEmpty(map.get("boilerEnergyEfficiencyCertificate")) ? null : JSONObject.toJSONString(map.get("boilerEnergyEfficiencyCertificate")));
//                idxBizJgFactoryInfoMapper.updateById(idxBizJgFactoryInfo);
//
//                // 使用信息
//                LambdaQueryWrapper<IdxBizJgUseInfo> lambda = new QueryWrapper<IdxBizJgUseInfo>().lambda();
//                lambda.eq(IdxBizJgUseInfo::getRecord, String.valueOf(map.get("equipId")));
//                IdxBizJgUseInfo useInfo = useInfoMapper.selectOne(lambda);
//                useInfo.setIsIntoManagement(Boolean.TRUE);
//                useInfo.setUseUnitCreditCode(String.valueOf(map.get("useUnitCreditCode")));
//                useInfo.setUseUnitName(String.valueOf(map.get("useUnitName")));
//                useInfo.setProvince("610000");
//                useInfo.setProvinceName("陕西省");
//                useInfo.setCity(String.valueOf(map.get("city")));
//                useInfo.setCityName(useCityName);
//                useInfo.setCounty(String.valueOf(map.get("county")));
//                useInfo.setCountyName(useCountyName);
//                useInfo.setFactoryUseSiteStreet(String.valueOf(map.get("factoryUseSiteStreet")));
//                useInfo.setStreetName(useStreetName);
//                useInfo.setAddress(String.valueOf(map.get("address")));
//                useInfo.setIsNotXiXian(String.valueOf(map.get("isXixian")));
//                useInfo.setEstateUnitCreditCode(String.valueOf(map.get("estateUnitName")).split("_")[0]);
//                useInfo.setEstateUnitName(String.valueOf(map.get("estateUnitName")).split("_")[1]);
//                useInfo.setPhone(String.valueOf(map.get("phone")));
//                useInfo.setSafetyManager(String.valueOf(map.get("safetyManager")).split("_")[1]);
//                useInfo.setUseDate(String.valueOf(map.get("useDate")));
//                useInfo.setLongitudeLatitude(JSON.toJSONString(map.get("longitudeLatitude")));
//                useInfo.setUsePlace((String) map.get("usePlace"));
//                useInfo.setOldUseRegistrationTable(JSON.toJSONString(map.get("oldUseRegistrationTable")));
//                useInfo.setOldUseRegistrationCertificate(JSON.toJSONString(map.get("oldUseRegistrationCertificate")));
//                useInfo.setORegDate(String.valueOf(map.get("oRegDate")));
//                useInfo.setORegUnit(String.valueOf(map.get("oRegUnit")));
//                // 历史平台登记，将设备状态改为在用
//                useInfo.setEquState(EquimentEnum.ZAIYONG.getCode().toString());
//                String usePlace = useInfo.getProvinceName() + "/" + useInfo.getCityName() + "/" + useInfo.getCountyName() + "/" + useInfo.getStreetName();
//                useInfoMapper.updateById(useInfo);
//
//                LambdaQueryWrapper<IdxBizJgOtherInfo> otherLambda = new QueryWrapper<IdxBizJgOtherInfo>().lambda();
//                otherLambda.eq(IdxBizJgOtherInfo::getRecord, map.get("equipId"));
//                IdxBizJgOtherInfo otherInfo = otherInfoMapper.selectOne(otherLambda);
//
//                // 检验检测【一对多，暂时只取最新一条数据】
//                IdxBizJgInspectionDetectionInfo inspectionDetectionInfo = iIdxBizJgInspectionDetectionInfoService.queryNewestDetailByRecord(String.valueOf(map.get("equipId")));
//                if (ObjectUtils.isEmpty(inspectionDetectionInfo.getSequenceNbr())) {
//                    inspectionDetectionInfo.setRecord((String) map.get("equipId"));
//                }
//                inspectionDetectionInfo.setInspectOrgCode(MapUtil.getStr(map, "inspectOrgCode"));
//                inspectionDetectionInfo.setInspectOrgName(MapUtil.getStr(map, "inspectOrgName"));
//                inspectionDetectionInfo.setInspectConclusion(MapUtil.getStr(map, "inspectConclusion"));
//                inspectionDetectionInfo.setInspectType(MapUtil.getStr(map, "inspectType"));
//                inspectionDetectionInfo.setInspectDate(!StringUtils.isEmpty(map.get("inspectDate")) ? DateUtil.parse(String.valueOf(map.get("inspectDate")), "yyyy-MM-dd") : null);
//                inspectionDetectionInfo.setInspectStaff(MapUtil.getStr(map, "inspectStaff"));
//                inspectionDetectionInfo.setNextInspectDate(!StringUtils.isEmpty(map.get("nextInspectDate")) ? DateUtil.parse(String.valueOf(map.get("nextInspectDate")), "yyyy-MM-dd") : null);
//                inspectionDetectionInfo.setInspectReport(JSONObject.toJSONString(map.get("inspectReport")));
//                inspectionDetectionInfo.setInspectReportNo(String.valueOf(map.get("inspectReportNo")));
//                iIdxBizJgInspectionDetectionInfoService.saveOrUpdateData(inspectionDetectionInfo);
//
//                // 生成监管码 96333码
//                this.createCode(map, jgUseRegistration, registerInfo, useInfo, otherInfo);
//                map.put("equCode", equCode);
//                // 更新es
//                this.updateEsData2(usePlace, map, otherInfo, useInfo, jgUseRegistration, map);
//                this.updateById(jgUseRegistration);
//
//                // 设备安装信息更新
//                this.historyEquUpdateInstallInfo(map);
//
//                // 设备维保信息更新,只有电梯
//                if ("3000".equals(map.get("equList")) || "3000".equals(map.get("equListCode"))) {
//                    this.historyEquUpdateMaintenanceInfo(map);
//                }
//
//                // 历史设备 生成证书管理表记录 & 生成安装 维保等操作记录
//                this.historyEquGenManageRelated(map, jgUseRegistration, registerInfo, idxBizJgFactoryInfo);
        }
        return null;
    }

    public void updateBizInfo(Map<String, Object> changeData) {
        JgUseRegistration useRegistration = JSON.parseObject(JSON.toJSONString(changeData), JgUseRegistration.class);
        String actionKey = StringUtils.isEmpty(useRegistration.getRegType()) ? "" : useRegistration.getRegType() + "-" + useRegistration.getManageType();
        Function<JSONObject, Boolean> h = bizHandlers.getOrDefault(actionKey, this::handleUseRegistrationEdit);
        h.apply((new JSONObject(changeData)));
    }

    public List<FieldChangeMeta> updEqData(String applyNo, JSONObject data) {
        JgUseRegistration jgUseRegistration = jgUseRegistrationService.getOne(new LambdaQueryWrapper<JgUseRegistration>().eq(JgUseRegistration::getApplyNo, applyNo));
        String useOrgCode = jgUseRegistration.getUseRegistrationCode();
        JSONObject hisData = jgUseRegistrationService.getHisData(jgUseRegistration);
        // todo 多线程时大量数据加载到内存，内存可能会爆 需要优化服务内只存一份
        // 市
        List<LinkedHashMap> city = (List<LinkedHashMap>) redisUtils.get("CITY");
        // 区
        List<LinkedHashMap> region = (List<LinkedHashMap>) redisUtils.get("REGION");
        // 街道
        List<LinkedHashMap> street = (List<LinkedHashMap>) redisUtils.get("STREET");
        IdxBizJgUseInfo useInfoOld = jgUseRegistrationService.getUseInfoMapper().selectOne(new LambdaQueryWrapper<IdxBizJgUseInfo>().eq(IdxBizJgUseInfo::getRecord, data.getString("equipId")));
        IdxBizJgUseInfo useInfoNew = BeanUtil.copyProperties(useInfoOld, IdxBizJgUseInfo.class);
        BeanUtil.copyProperties(hisData, useInfoNew);
        useInfoNew.setIsNotXiXian(hisData.getString("isXixian"));
        // 城市
        if (!ObjectUtils.isEmpty(hisData.get("city")) && !ObjectUtils.isEmpty(city)) {
            city.forEach(item -> {
                if (String.valueOf(item.get("regionCode")).equals(String.valueOf(hisData.get("city")))) {
                    useInfoNew.setCityName(String.valueOf(item.get("regionName")));
                }
            });
        }
        // 区县
        if (!ObjectUtils.isEmpty(hisData.get("county")) && !ObjectUtils.isEmpty(region)) {
            region.forEach(item -> {
                if (String.valueOf(item.get("regionCode")).equals(String.valueOf(hisData.get("county")))) {
                    useInfoNew.setCountyName(String.valueOf(item.get("regionName")));
                }
            });
        }
        // 街道
        if (!ObjectUtils.isEmpty(hisData.get("factoryUseSiteStreet")) && !ObjectUtils.isEmpty(street)) {
            street.forEach(item -> {
                if (String.valueOf(item.get("regionCode")).equals(String.valueOf(hisData.get("factoryUseSiteStreet")))) {
                    useInfoNew.setStreetName(String.valueOf(item.get("regionName")));
                }
            });
        }
        // 安全管理员名称
        useInfoNew.setSafetyManager(hisData.getString("safetyManagerName"));
        useInfoNew.setProvince("610000");
        useInfoNew.setProvinceName("陕西省");
        // 产权单位信息
        if (hisData.containsKey("estateUnitName")) {
            String[] estateUnit = String.valueOf(hisData.getString("estateUnitName")).split("_");
            useInfoNew.setEstateUnitCreditCode(estateUnit[0]);
            useInfoNew.setEstateUnitName(estateUnit[1]);
        }
        // 更新使用信息及日志比较生成
        List<FieldChangeMeta> useInfoFieldChangeMetas = commonEquipDataProcessService.simpleTrackAndUpdate(jgUseRegistrationService.getUseInfoMapper(), useInfoOld, useInfoNew, useInfoNew.getRecord(), "record", useInfoNew.getRecord());
        List<FieldChangeMeta> fieldChangeMetas = new ArrayList<>(useInfoFieldChangeMetas);
        // todo 更新检验检测信息 原有业务系统要改造在审核完成时更新json时 把jySeq写上
        IdxBizJgInspectionDetectionInfo inspectionDetectionInfoOld = jgUseRegistrationService.getInspectionDetectionInfoService().getById(hisData.getLong("jySeq"));
        if (inspectionDetectionInfoOld == null) {
            inspectionDetectionInfoOld = jgUseRegistrationService.getInspectionDetectionInfoService().queryNewestDetailByRecord(useInfoNew.getRecord());
        }
        IdxBizJgInspectionDetectionInfo inspectionDetectionInfoNew = BeanUtil.copyProperties(inspectionDetectionInfoOld, IdxBizJgInspectionDetectionInfo.class);
        String jySeq = inspectionDetectionInfoNew.getSequenceNbr();
        BeanUtil.copyProperties(hisData, inspectionDetectionInfoNew);
        // TODO 重要 json里有主表的主键copyProperties时导致inspectionDetectionInfoN的主键被覆盖 故如此赋值
        inspectionDetectionInfoNew.setSequenceNbr(jySeq);
        List<FieldChangeMeta> inspectionFieldChangeMetas = commonEquipDataProcessService.simpleTrackAndUpdate(jgUseRegistrationService.getInspectionDetectionInfoService().getBaseMapper(), inspectionDetectionInfoOld, inspectionDetectionInfoNew, useInfoNew.getRecord(), "SEQUENCE_NBR", inspectionDetectionInfoOld.getSequenceNbr());
        fieldChangeMetas.addAll(inspectionFieldChangeMetas);
        // 更新注册登记信息表
        IdxBizJgRegisterInfo registerInfoOld = jgUseRegistrationService.getIdxBizJgRegisterInfoService().getOne(new LambdaQueryWrapper<IdxBizJgRegisterInfo>().eq(IdxBizJgRegisterInfo::getRecord, data.getString("equipId")));
        IdxBizJgRegisterInfo registerInfoNew = BeanUtil.copyProperties(registerInfoOld, IdxBizJgRegisterInfo.class);
        registerInfoNew.setCylinderCategory(hisData.getString("cylinderCategory"));
        registerInfoNew.setUseOrgCode(useOrgCode);
        List<FieldChangeMeta> registerInfoFieldChangeMetas = commonEquipDataProcessService.simpleTrackAndUpdate(jgUseRegistrationService.getIdxBizJgRegisterInfoService().getBaseMapper(), registerInfoOld, registerInfoNew, useInfoNew.getRecord(), "record", useInfoNew.getRecord());
        fieldChangeMetas.addAll(registerInfoFieldChangeMetas);
        // 更新设备监管部门
        if (hisData.containsKey("orgBranchCode") && !ObjectUtils.isEmpty(hisData.get("orgBranchCode"))) {
            IdxBizJgSupervisionInfo idxBizJgSupervisionInfoOld = jgUseRegistrationService.getIdxBizJgSupervisionInfoMapper().selectOne(new LambdaQueryWrapper<IdxBizJgSupervisionInfo>().eq(IdxBizJgSupervisionInfo::getRecord, data.getString("equipId")));
            IdxBizJgSupervisionInfo idxBizJgSupervisionInfNew = BeanUtil.copyProperties(idxBizJgSupervisionInfoOld, IdxBizJgSupervisionInfo.class);
            String[] orgBranch = String.valueOf(hisData.getString("orgBranchCode")).split("_");
            idxBizJgSupervisionInfNew.setOrgBranchCode(orgBranch[0]);
            idxBizJgSupervisionInfNew.setOrgBranchName(orgBranch[1]);
            idxBizJgSupervisionInfNew.setCompanyOrgBranchCode(commonMapper.geCompanyCodeByOrgCode(orgBranch[0]));
            List<FieldChangeMeta> supervisionInfoFieldChangeMetas = commonEquipDataProcessService.simpleTrackAndUpdate(jgUseRegistrationService.getIdxBizJgSupervisionInfoMapper(), idxBizJgSupervisionInfoOld, idxBizJgSupervisionInfNew, useInfoNew.getRecord(), "record", useInfoNew.getRecord());
            fieldChangeMetas.addAll(supervisionInfoFieldChangeMetas);
        }
        // todo 历史无证电梯维保信息可编辑 原有业务系统要改造在审核完成时更新json时 把维保流水seq写上 用来唯一一条更新
        if ("2".equals(jgUseRegistration.getRegType()) && "3000".equals(hisData.getString("equList"))) {
            IdxBizJgMaintenanceRecordInfo idxBizJgMaintenanceRecordInfoOld = jgUseRegistrationService.getIdxBizJgMaintenanceRecordInfoService().queryNewestDetailByRecord(useInfoNew.getRecord());
            // 电梯处理维保信息
            IdxBizJgMaintenanceRecordInfo idxBizJgMaintenanceRecordInfoNew = new IdxBizJgMaintenanceRecordInfo();
            BeanUtil.copyProperties(idxBizJgMaintenanceRecordInfoOld, idxBizJgMaintenanceRecordInfoNew);
            BeanUtil.copyProperties(jgUseRegistrationService.buildIdxBizJgMaintenanceRecordInfo(hisData), idxBizJgMaintenanceRecordInfoNew);
            idxBizJgMaintenanceRecordInfoNew.setSequenceNbr(idxBizJgMaintenanceRecordInfoOld.getSequenceNbr());
            idxBizJgMaintenanceRecordInfoNew.setRecord(idxBizJgMaintenanceRecordInfoOld.getRecord());
            List<FieldChangeMeta> supervisionInfoFieldChangeMetas = commonEquipDataProcessService.simpleTrackAndUpdate(jgUseRegistrationService.getIdxBizJgMaintenanceRecordInfoService().getBaseMapper(), idxBizJgMaintenanceRecordInfoOld, idxBizJgMaintenanceRecordInfoNew, useInfoNew.getRecord(), "SEQUENCE_NBR", idxBizJgMaintenanceRecordInfoOld.getSequenceNbr());
            fieldChangeMetas.addAll(supervisionInfoFieldChangeMetas);
            jgUseRegistrationService.historyEquUpdateMaintenanceInfo(hisData);
        }
        // todo 历史无证可编辑安装信息 安装信息写入后需要把 流水表的主键写回json 用来唯一一条更新
        if ("2".equals(jgUseRegistration.getRegType())) {
            IdxBizJgConstructionInfo constructionInfoOld = jgUseRegistrationService.getIdxBizJgConstructionInfoService().queryNewestDetailByRecord(String.valueOf(hisData.get("equipId")));
            IdxBizJgConstructionInfo constructionInfoNew = BeanUtil.copyProperties(constructionInfoOld, IdxBizJgConstructionInfo.class);
            if (!ValidationUtil.isEmpty(constructionInfoOld.getSequenceNbr())) {
                constructionInfoNew.setUscUnitName(ValidationUtil.isEmpty(hisData.get("uscUnitName")) ? null : String.valueOf(hisData.get("uscUnitName")));
                constructionInfoNew.setConstructionLeaderName(ValidationUtil.isEmpty(hisData.get("installLeaderId")) ? null : String.valueOf(hisData.get("installLeaderId")));
                constructionInfoNew.setConstructionLeaderPhone(ValidationUtil.isEmpty(hisData.get("installLeaderPhone")) ? null : String.valueOf(hisData.get("installLeaderPhone")));
                constructionInfoNew.setProxyStatementAttachment(ValidationUtil.isEmpty(hisData.get("proxyStatementAttachmentList")) ? null : JSONObject.toJSONString(hisData.get("proxyStatementAttachmentList")));
                constructionInfoNew.setConstructionContractAttachment(ValidationUtil.isEmpty(hisData.get("installContractAttachment")) ? null : JSONObject.toJSONString(hisData.get("installContractAttachment")));
                constructionInfoNew.setConstructionOtherAccessories(ValidationUtil.isEmpty(hisData.get("insOtherAccessories")) ? null : JSONObject.toJSONString(hisData.get("insOtherAccessories")));
                try {
                    constructionInfoNew.setUscDate(ValidationUtil.isEmpty(hisData.get("installStartDate")) ? null : DateUtils.dateParse((String) hisData.get("installStartDate"), DateUtils.DATE_PATTERN));
                } catch (ParseException e) {
                    log.error(e.getMessage(), e);
                    throw new BadRequest("安装日期时间转化错误");
                }
                List<FieldChangeMeta> constructionInfoFieldChangeMetas = commonEquipDataProcessService.simpleTrackAndUpdate(jgUseRegistrationService.getIdxBizJgConstructionInfoService().getBaseMapper(), constructionInfoOld, constructionInfoNew, useInfoNew.getRecord(), "SEQUENCE_NBR", constructionInfoOld.getSequenceNbr());
                fieldChangeMetas.addAll(constructionInfoFieldChangeMetas);
            }
        }
        return fieldChangeMetas;
    }


}
