package com.yeejoin.amos.boot.module.jg.biz.edit.process.equip;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.yeejoin.amos.boot.biz.common.annotation.FieldDisplayDefine;
import com.yeejoin.amos.boot.biz.common.annotation.Group;
import com.yeejoin.amos.boot.module.common.api.dto.IBaseChangeData;
import com.yeejoin.amos.boot.module.jg.api.dto.*;
import com.yeejoin.amos.boot.module.jg.biz.service.*;
import com.yeejoin.amos.boot.module.jg.biz.service.impl.IdxBizJgUseInfoServiceImpl;
import com.yeejoin.amos.boot.module.jg.biz.service.impl.JgUseRegistrationServiceImpl;
import com.yeejoin.amos.boot.module.ymt.api.entity.*;
import com.yeejoin.amos.boot.module.ymt.api.enums.EquipmentClassifityEnum;
import com.yeejoin.amos.boot.module.ymt.api.mapper.IdxBizJgDesignInfoMapper;
import com.yeejoin.amos.boot.module.ymt.api.mapper.IdxBizJgFactoryInfoMapper;
import com.yeejoin.amos.boot.module.ymt.api.mapper.IdxBizJgOtherInfoMapper;
import com.yeejoin.amos.boot.module.ymt.api.mapper.IdxBizJgRegisterInfoMapper;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.lang.reflect.Field;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.yeejoin.amos.boot.module.jg.biz.edit.core.SubClassFinder.getAllSubClasses;
import static com.yeejoin.amos.boot.module.jg.biz.edit.utils.ReflectiveFieldAccessor.getAllFields;

@Component
@RequiredArgsConstructor
public class CommonEquipDataProcessService {

    private final IdxBizJgDesignInfoMapper idxBizJgDesignInfoMapper;

    private final IdxBizJgFactoryInfoMapper jgFactoryInfoMapper;

    private final IdxBizJgRegisterInfoMapper idxBizJgRegisterInfoMapper;

    private final IIdxBizJgTechParamsElevatorService iIdxBizJgTechParamsElevatorService;

    private final IIdxBizJgTechParamsVehicleService iIdxBizJgTechParamsVehicleService;

    private final IIdxBizJgTechParamsRopewayService iIdxBizJgTechParamsRopewayService;

    private final IIdxBizJgTechParamsRidesService iIdxBizJgTechParamsRidesService;

    private final IIdxBizJgTechParamsBoilerService iIdxBizJgTechParamsBoilerService;

    private final IIdxBizJgTechParamsVesselService iIdxBizJgTechParamsVesselService;

    private final IIdxBizJgTechParamsPipelineService iIdxBizJgTechParamsPipelineService;

    private final IIdxBizJgTechParamsLiftingService iIdxBizJgTechParamsLiftingService;

    private final EquipChangeDataUpdateServiceImpl equipChangeDataUpdateServiceImpl;

    private final IdxBizJgUseInfoServiceImpl idxBizJgUseInfoService;

    private final JgUseRegistrationServiceImpl jgUseRegistrationService;

    private final List<String> jsonFields = new ArrayList<>();

    private final IdxBizJgOtherInfoMapper idxBizJgOtherInfoMapper;

    @PostConstruct
    public void init() {
        Set<Class<? extends IBaseChangeData>> subClasses = getAllSubClasses();
        for (Class<? extends IBaseChangeData> subClass : subClasses) {
            Field[] fields = subClass.getDeclaredFields();
            for (Field field : fields) {
                field.setAccessible(true);
                FieldDisplayDefine fieldDisplayDefine = field.getAnnotation(FieldDisplayDefine.class);
                if (fieldDisplayDefine != null && fieldDisplayDefine.type() == JSON.class) {
                    jsonFields.add(field.getName());
                }
            }
        }
    }

    public static <T extends BaseChangeDataDto> T castMap2Bean(Map<String, Object> map, Class<T> clazz) {
        try {
            T t = clazz.newInstance();
            BeanUtil.copyProperties(JSON.parse(JSONObject.toJSONString(map)), t, true);
            return t;
        } catch (InstantiationException | IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }


    public void checkValidField(String record, BaseChangeDataDto changeDataDto, String equList, String equCategory, String equDefine) {
        // 注册信息
        if (changeDataDto instanceof EquipRegisterChangeDataDto) {
            EquipRegisterChangeDataDto registerChangeDataDto = (EquipRegisterChangeDataDto) changeDataDto;
            equipChangeDataUpdateServiceImpl.checkRegisterChangeData(record, registerChangeDataDto);
        }
        // 制造信息
        if (changeDataDto instanceof EquipFactoryChangeDataDto) {
            EquipFactoryChangeDataDto equipFactoryChangeDataDto = (EquipFactoryChangeDataDto) changeDataDto;
            equipChangeDataUpdateServiceImpl.checkFactoryChangeData(record, equipFactoryChangeDataDto, equList, equCategory, equDefine);
        }
        // 设计信息
        if (changeDataDto instanceof EquipDesignChangeDataDto) {
            EquipDesignChangeDataDto equipDesignChangeDataDto = (EquipDesignChangeDataDto) changeDataDto;
            equipChangeDataUpdateServiceImpl.checkDesignChangeData(record, equipDesignChangeDataDto);
        }
    }


    public void buildChangeFields(String record, EquipDesignChangeDataDto equipDesignChangeDataDto, EquipFactoryChangeDataDto equipFactoryChangeDataDto, EquipRegisterChangeDataDto registerChangeDataDto, EquipOtherInfoChangeDataDto otherInfoChangeDataDto, EquipUseInfoChangeDataDto useInfoChangeDataDto, List<FieldChangeMeta> allChangeColumns) {
        // 构造新对象行转列
        List<FieldChangeMeta> designInfoNew = this.buildDesignInfoNew(equipDesignChangeDataDto, record);
        List<FieldChangeMeta> factoryInfoNew = this.buildFactoryInfoNew(equipFactoryChangeDataDto, record);
        List<FieldChangeMeta> registerInfoNew = this.buildRegisterInfoNew(registerChangeDataDto, record);
        if (useInfoChangeDataDto != null) {
            List<FieldChangeMeta> useInfoNew = convertBeanField2Column2(useInfoChangeDataDto, record);
            List<FieldChangeMeta> useInfoOld = this.buildUseInfoOld(record);
            allChangeColumns.addAll(mergeChangeFields(useInfoNew, useInfoOld));
        }
        if (otherInfoChangeDataDto != null) {
            List<FieldChangeMeta> otherInfoNew = convertBeanField2Column2(otherInfoChangeDataDto, record);
            List<FieldChangeMeta> otherInfoOld = this.buildOtherInfoOld(record);
            allChangeColumns.addAll(mergeChangeFields(otherInfoNew, otherInfoOld));

        }
        // 老数据查询行转列
        List<FieldChangeMeta> designInfoOld = this.buildDesignInfoOld(record);
        List<FieldChangeMeta> factoryInfoOld = this.buildFactoryInfoOld(record);
        List<FieldChangeMeta> registerInfoOld = this.buildRegisterInfoOld(record);
        allChangeColumns.addAll(mergeChangeFields(designInfoNew, designInfoOld));
        allChangeColumns.addAll(mergeChangeFields(factoryInfoNew, factoryInfoOld));
        allChangeColumns.addAll(mergeChangeFields(registerInfoNew, registerInfoOld));
    }

    public void buildChangeFields(String record, EquipDesignChangeDataDto equipDesignChangeDataDto, EquipFactoryChangeDataDto equipFactoryChangeDataDto, EquipRegisterChangeDataDto registerChangeDataDto, EquipUseInfoChangeDataDto useInfoChangeDataDto, List<FieldChangeMeta> allChangeColumns) {
        // 构造新对象行转列
        List<FieldChangeMeta> designInfoNew = convertBeanField2Column2(equipDesignChangeDataDto, record);
        List<FieldChangeMeta> factoryInfoNew = convertBeanField2Column2(equipFactoryChangeDataDto, record);
        List<FieldChangeMeta> registerInfoNew = convertBeanField2Column2(registerChangeDataDto, record);
        List<FieldChangeMeta> useInfoChangeNew = convertBeanField2Column2(useInfoChangeDataDto, record);
        // 老数据查询行转列
        List<FieldChangeMeta> designInfoOld = this.buildDesignInfoOld(record);
        List<FieldChangeMeta> factoryInfoOld = this.buildFactoryInfoOld(record);
        List<FieldChangeMeta> registerInfoOld = this.buildRegisterInfoOld(record);
        List<FieldChangeMeta> useInfoOld = this.buildUseInfoOld(record);
        allChangeColumns.addAll(mergeChangeFields(designInfoNew, designInfoOld));
        allChangeColumns.addAll(mergeChangeFields(factoryInfoNew, factoryInfoOld));
        allChangeColumns.addAll(mergeChangeFields(registerInfoNew, registerInfoOld));
        allChangeColumns.addAll(mergeChangeFields(useInfoChangeNew, useInfoOld));
    }

    /**
     * 业务处理-业务对象
     *
     * @param record        设备主键
     * @param changeDataDto 变更后数据
     */
    public void dealBizDataForEquip(String record, BaseChangeDataDto changeDataDto) {
        // 注册信息
        if (changeDataDto instanceof EquipRegisterChangeDataDto) {
            EquipRegisterChangeDataDto registerChangeDataDto = (EquipRegisterChangeDataDto) changeDataDto;
            equipChangeDataUpdateServiceImpl.updateRegisterChangeData(record, registerChangeDataDto);
        }
        // 制造信息
        if (changeDataDto instanceof EquipFactoryChangeDataDto) {
            EquipFactoryChangeDataDto equipFactoryChangeDataDto = (EquipFactoryChangeDataDto) changeDataDto;
            equipChangeDataUpdateServiceImpl.updateFactoryChangeData(record, equipFactoryChangeDataDto);
        }
        // 设计信息
        if (changeDataDto instanceof EquipDesignChangeDataDto) {
            EquipDesignChangeDataDto equipDesignChangeDataDto = (EquipDesignChangeDataDto) changeDataDto;
            equipChangeDataUpdateServiceImpl.updateDesignChangeData(record, equipDesignChangeDataDto);
        }
        // 使用信息
        if (changeDataDto instanceof EquipUseInfoChangeDataDto) {
            EquipUseInfoChangeDataDto equipUseInfoChangeDataDto = (EquipUseInfoChangeDataDto) changeDataDto;
            equipChangeDataUpdateServiceImpl.updateUseInfoChangeData(record, equipUseInfoChangeDataDto);
        }
        // 检验信息
        if (changeDataDto instanceof EquipInspectChangeDataDto) {
            EquipInspectChangeDataDto equipUseInfoChangeDataDto = (EquipInspectChangeDataDto) changeDataDto;
            equipChangeDataUpdateServiceImpl.updateOrSaveInspectInfo(record, equipUseInfoChangeDataDto);
        }
        // 气瓶钢印信息
        if (changeDataDto instanceof EquipOtherInfoChangeDataDto) {
            EquipOtherInfoChangeDataDto otherInfoChangeDataDto = (EquipOtherInfoChangeDataDto) changeDataDto;
            equipChangeDataUpdateServiceImpl.updateOtherInfoChangeData(record, otherInfoChangeDataDto);
        }

    }

    private String buildTableName(Class<?> clazz) {
        Group group = clazz.getAnnotation(Group.class);
        return String.format("\"%s_%s\"", "idx_biz_jg", group.value());
    }


    public static List<FieldChangeMeta> mergeChangeFields(List<FieldChangeMeta> changeDataNew, List<FieldChangeMeta> changeDataOld) {
        Map<String, FieldChangeMeta> oldKv = changeDataOld.stream().collect(Collectors.toMap(FieldChangeMeta::getColumnKey, Function.identity()));
        changeDataNew.forEach(o -> {
            // 正常不会出现此情况，old数据 大于等于 new 数据的key
            if (oldKv.get(o.getColumnKey()) != null) {
                o.setColumnOldValue(oldKv.get(o.getColumnKey()).getColumnOldValue());
            }
        });
        return changeDataNew.stream().filter(n -> !StringUtils.equals(n.getColumnNewValue(), n.getColumnOldValue())).collect(Collectors.toList());
    }

    private static TechParamsBoilerChangeFieldDto getTechParamsBoilerChangeField(Map<String, Object> changeData) {
        TechParamsBoilerChangeFieldDto newBoiler = new TechParamsBoilerChangeFieldDto();
        BeanUtil.copyProperties(JSON.parse(JSONObject.toJSONString(changeData)), newBoiler, true);
        return newBoiler;
    }

    private TechParamsBoilerChangeFieldDto getTechParamsBoilerChangeField(String record) {
        IdxBizJgTechParamsBoiler boiler = iIdxBizJgTechParamsBoilerService.getOneData(record);
        TechParamsBoilerChangeFieldDto oldBoiler = new TechParamsBoilerChangeFieldDto();
        BeanUtil.copyProperties(boiler, oldBoiler);
        return oldBoiler;
    }

    BaseTechParamsFieldDto buildTechParamInfo(EquipRegisterChangeDataDto equipRegisterChangeDataDto, String record) {
        EquipmentClassifityEnum equipmentClassifityEnum = EquipmentClassifityEnum.getOne(equipRegisterChangeDataDto.getEquList());
        switch (Objects.requireNonNull(equipmentClassifityEnum)) {
            case GL:
                // 锅炉
                IdxBizJgTechParamsBoiler boiler = iIdxBizJgTechParamsBoilerService.getOneData(record);
                // 技术参数对象转换为全部技术参数 ，其他无用字段去掉
                return BeanUtil.copyProperties(boiler, TechParamsBoilerChangeFieldDto.class);
            case YLRQ:
                // 压力容器
                IdxBizJgTechParamsVessel techParamsVessel = iIdxBizJgTechParamsVesselService.getOneData(record);
                // 技术参数对象转换为全部技术参数 ，其他无用字段去掉
                return BeanUtil.copyProperties(techParamsVessel, TechParamsVesselChangeFieldDto.class);
            case DT:
                // 电梯
                IdxBizJgTechParamsElevator techParamsElevator = iIdxBizJgTechParamsElevatorService.getOneData(record);
                // 技术参数对象转换为全部技术参数 ，其他无用字段去掉
                return BeanUtil.copyProperties(techParamsElevator, TechParamsElevatorChangeFieldDto.class);
            case QZJX:
                // 起重机械
                IdxBizJgTechParamsLifting techParamsLifting = iIdxBizJgTechParamsLiftingService.getOneData(record);
                // 技术参数对象转换为全部技术参数 ，其他无用字段去掉
                return BeanUtil.copyProperties(techParamsLifting, TechParamsLiftingChangeFieldDto.class);
            case CC:
                // 场（厂）内专用机动车辆
                IdxBizJgTechParamsVehicle techParamsVehicle = iIdxBizJgTechParamsVehicleService.getOneData(record);
                // 技术参数对象转换为全部技术参数 ，其他无用字段去掉
                return BeanUtil.copyProperties(techParamsVehicle, TechParamsVehicleChangeFieldDto.class);
            case YLSS:
                // 大型游乐设施
                IdxBizJgTechParamsRides techParamsRides = iIdxBizJgTechParamsRidesService.getOneData(record);
                // 技术参数对象转换为全部技术参数 ，其他无用字段去掉
                return BeanUtil.copyProperties(techParamsRides, TechParamsRidesChangeFieldDto.class);
            case YLGD:
                // 压力管道
                IdxBizJgTechParamsPipeline techParamsPipeline = iIdxBizJgTechParamsPipelineService.getOneData(record);
                // 技术参数对象转换为全部技术参数 ，其他无用字段去掉
                return BeanUtil.copyProperties(techParamsPipeline, TechParamsPipelineChangeFieldDto.class);
            case KYSD:
                // 客运索道
                IdxBizJgTechParamsRopeway techParamsRopeway = iIdxBizJgTechParamsRopewayService.getOneData(record);
                // 技术参数对象转换为全部技术参数 ，其他无用字段去掉
                return BeanUtil.copyProperties(techParamsRopeway, TechParamsRopewayChangeFieldDto.class);
            default:
                break;
        }
        return null;
    }


    void updateTechParamInfo(EquipRegisterChangeDataDto equipRegisterChangeDataDto, String record, Map<String, Object> changeData, List<FieldChangeMeta> allChange) {
        EquipmentClassifityEnum equipmentClassifityEnum = EquipmentClassifityEnum.getOne(equipRegisterChangeDataDto.getEquList());
        switch (Objects.requireNonNull(equipmentClassifityEnum)) {
            case GL:
                // 锅炉
                TechParamsBoilerChangeFieldDto newBoiler = getTechParamsBoilerChangeField(changeData);
                TechParamsBoilerChangeFieldDto oldBoiler = getTechParamsBoilerChangeField(record);
                // 字段行转列
                List<FieldChangeMeta> boilerChangeDataNew = convertBeanField2Column2(newBoiler, record);
                List<FieldChangeMeta> boilerChangeDataOld = convertBeanField2Column2(oldBoiler, record);
                // 比对
                List<FieldChangeMeta> boilerChangeFields = mergeChangeFields(boilerChangeDataNew, boilerChangeDataOld);
                // 业务处理
                equipChangeDataUpdateServiceImpl.updateTechParamByRecord(this.buildTableName(TechParamsBoilerChangeFieldDto.class), record, boilerChangeFields);
                // 日志数据记录
                allChange.addAll(boilerChangeDataNew);
                break;
            case YLRQ:
                // 压力容器
                TechParamsVesselChangeFieldDto newVessel = new TechParamsVesselChangeFieldDto();
                BeanUtil.copyProperties(JSON.parse(JSONObject.toJSONString(changeData)), newVessel, true);
                IdxBizJgTechParamsVessel techParamsVessel = iIdxBizJgTechParamsVesselService.getOneData(record);
                TechParamsVesselChangeFieldDto oldVessel = BeanUtil.copyProperties(techParamsVessel, TechParamsVesselChangeFieldDto.class);
                // 字段行转列
                List<FieldChangeMeta> newVesselChangeData = convertBeanField2Column2(newVessel, record);
                List<FieldChangeMeta> oldVesselChangeData = convertBeanField2Column2(oldVessel, record);
                // 比对
                List<FieldChangeMeta> vesselChangeFields = mergeChangeFields(newVesselChangeData, oldVesselChangeData);
                // 业务处理
                equipChangeDataUpdateServiceImpl.updateTechParamByRecord(this.buildTableName(TechParamsVesselChangeFieldDto.class), record, vesselChangeFields);
                // 日志数据记录
                allChange.addAll(vesselChangeFields);
                break;
            case DT:
                // 电梯
                TechParamsElevatorChangeFieldDto newElevator = new TechParamsElevatorChangeFieldDto();
                BeanUtil.copyProperties(JSON.parse(JSONObject.toJSONString(changeData)), newElevator, true);

                IdxBizJgTechParamsElevator techParamsElevator = iIdxBizJgTechParamsElevatorService.getOneData(record);
                TechParamsElevatorChangeFieldDto oldElevator = BeanUtil.copyProperties(techParamsElevator, TechParamsElevatorChangeFieldDto.class);
                // 字段行转列
                List<FieldChangeMeta> newElevatorChangeData = convertBeanField2Column2(newElevator, record);
                List<FieldChangeMeta> oldElevatorChangeData = convertBeanField2Column2(oldElevator, record);
                // 比对
                List<FieldChangeMeta> elevatorChangeFields = mergeChangeFields(newElevatorChangeData, oldElevatorChangeData);
                // 业务处理
                equipChangeDataUpdateServiceImpl.updateTechParamByRecord(this.buildTableName(TechParamsElevatorChangeFieldDto.class), record, elevatorChangeFields);
                // 日志数据记录
                allChange.addAll(elevatorChangeFields);
                break;
            case QZJX:
                // 起重机械
                TechParamsLiftingChangeFieldDto newLifting = new TechParamsLiftingChangeFieldDto();
                BeanUtil.copyProperties(JSON.parse(JSONObject.toJSONString(changeData)), newLifting, true);

                IdxBizJgTechParamsLifting techParamsLifting = iIdxBizJgTechParamsLiftingService.getOneData(record);
                // 技术参数对象转换为全部技术参数 ，其他无用字段去掉
                TechParamsLiftingChangeFieldDto oldLifting = BeanUtil.copyProperties(techParamsLifting, TechParamsLiftingChangeFieldDto.class);
                // 字段行转列
                List<FieldChangeMeta> newLiftingChangeData = convertBeanField2Column2(newLifting, record);
                List<FieldChangeMeta> oldLiftingChangeData = convertBeanField2Column2(oldLifting, record);
                // 比对
                List<FieldChangeMeta> liftingChangeFields = mergeChangeFields(newLiftingChangeData, oldLiftingChangeData);
                // 业务处理
                equipChangeDataUpdateServiceImpl.updateTechParamByRecord(this.buildTableName(TechParamsLiftingChangeFieldDto.class), record, liftingChangeFields);
                // 日志数据记录
                allChange.addAll(liftingChangeFields);
                break;
            case CC:
                // 场（厂）内专用机动车辆
                TechParamsVehicleChangeFieldDto newVehicle = new TechParamsVehicleChangeFieldDto();
                BeanUtil.copyProperties(JSON.parse(JSONObject.toJSONString(changeData)), newVehicle, true);

                IdxBizJgTechParamsVehicle techParamsVehicle = iIdxBizJgTechParamsVehicleService.getOneData(record);
                TechParamsVehicleChangeFieldDto oldVehicle = BeanUtil.copyProperties(techParamsVehicle, TechParamsVehicleChangeFieldDto.class);
                // 字段行转列
                List<FieldChangeMeta> newVehicleChangeData = convertBeanField2Column2(newVehicle, record);
                List<FieldChangeMeta> oldVehicleChangeData = convertBeanField2Column2(oldVehicle, record);
                // 比对
                List<FieldChangeMeta> vehicleChangeFields = mergeChangeFields(newVehicleChangeData, oldVehicleChangeData);
                // 业务处理
                equipChangeDataUpdateServiceImpl.updateTechParamByRecord(this.buildTableName(TechParamsVehicleChangeFieldDto.class), record, vehicleChangeFields);
                // 日志数据记录
                allChange.addAll(vehicleChangeFields);
                break;
            case YLSS:
                // 大型游乐设施
                TechParamsRidesChangeFieldDto newRides = new TechParamsRidesChangeFieldDto();
                BeanUtil.copyProperties(JSON.parse(JSONObject.toJSONString(changeData)), newRides, true);

                IdxBizJgTechParamsRides techParamsRides = iIdxBizJgTechParamsRidesService.getOneData(record);
                // 技术参数对象转换为全部技术参数 ，其他无用字段去掉
                TechParamsRidesChangeFieldDto oldRides = BeanUtil.copyProperties(techParamsRides, TechParamsRidesChangeFieldDto.class);
                // 字段行转列
                List<FieldChangeMeta> newRidesChangeData = convertBeanField2Column2(newRides, record);
                List<FieldChangeMeta> oldRidesChangeData = convertBeanField2Column2(oldRides, record);
                // 比对
                List<FieldChangeMeta> ridesChangeFields = mergeChangeFields(newRidesChangeData, oldRidesChangeData);
                // 业务处理
                equipChangeDataUpdateServiceImpl.updateTechParamByRecord(this.buildTableName(TechParamsRidesChangeFieldDto.class), record, ridesChangeFields);
                // 日志数据记录
                allChange.addAll(ridesChangeFields);
                break;
            case YLGD:
                // 压力管道
                TechParamsPipelineChangeFieldDto newPipeline = new TechParamsPipelineChangeFieldDto();
                BeanUtil.copyProperties(JSON.parse(JSONObject.toJSONString(changeData)), newPipeline, true);
                savePieLineTechParam(record, allChange, newPipeline);
                break;
            case KYSD:
                // 客运索道
                TechParamsRopewayChangeFieldDto newRopeway = new TechParamsRopewayChangeFieldDto();
                BeanUtil.copyProperties(JSON.parse(JSONObject.toJSONString(changeData)), newRopeway, true);

                IdxBizJgTechParamsRopeway techParamsRopeway = iIdxBizJgTechParamsRopewayService.getOneData(record);
                // 技术参数对象转换为全部技术参数 ，其他无用字段去掉
                TechParamsRopewayChangeFieldDto oldRopeway = BeanUtil.copyProperties(techParamsRopeway, TechParamsRopewayChangeFieldDto.class);
                // 字段行转列
                List<FieldChangeMeta> newRopewayChangeData = convertBeanField2Column2(newRopeway, record);
                List<FieldChangeMeta> oldRopewayChangeData = convertBeanField2Column2(oldRopeway, record);
                // 比对
                List<FieldChangeMeta> ropewayChangeFields = mergeChangeFields(newRopewayChangeData, oldRopewayChangeData);
                // 业务处理
                equipChangeDataUpdateServiceImpl.updateTechParamByRecord(this.buildTableName(TechParamsRopewayChangeFieldDto.class), record, ropewayChangeFields);
                // 日志数据记录
                allChange.addAll(ropewayChangeFields);
                break;
            default:
                break;
        }
    }

    public void savePieLineTechParam(String record, List<FieldChangeMeta> allChange, TechParamsPipelineChangeFieldDto newPipeline) {
        IdxBizJgTechParamsPipeline techParamsPipeline = iIdxBizJgTechParamsPipelineService.getOneData(record);
        // 技术参数对象转换为全部技术参数 ，其他无用字段去掉
        TechParamsPipelineChangeFieldDto oldPipeline = BeanUtil.copyProperties(techParamsPipeline, TechParamsPipelineChangeFieldDto.class);
        // 字段行转列
        List<FieldChangeMeta> newPipelineChangeData = convertBeanField2Column2(newPipeline, record);
        List<FieldChangeMeta> oldPipelineChangeData = convertBeanField2Column2(oldPipeline, record);
        // 比对
        List<FieldChangeMeta> pipelineChangeFields = mergeChangeFields(newPipelineChangeData, oldPipelineChangeData);
        // 业务处理
        equipChangeDataUpdateServiceImpl.updateTechParamByRecord(this.buildTableName(TechParamsPipelineChangeFieldDto.class), record, pipelineChangeFields);
        // 日志数据记录
        allChange.addAll(pipelineChangeFields);
    }

    public static List<FieldChangeMeta> convertBeanField2Column2(Object beanData, String changeId) {
        List<FieldChangeMeta> changeData = new ArrayList<>();
        Group group = beanData.getClass().getAnnotation(Group.class);
        Field[] fields = beanData.getClass().getDeclaredFields();
        for (Field field : fields) {
            field.setAccessible(true);
            FieldDisplayDefine displayDefine = field.getAnnotation(FieldDisplayDefine.class);
            try {
                if (displayDefine != null && displayDefine.isExist()) {
                    String fieldName = displayDefine.value();
                    FieldChangeMeta fieldChangeMeta = new FieldChangeMeta();
                    fieldChangeMeta.setColumnKey(field.getName());
                    fieldChangeMeta.setColumnFamily(group.value());
                    fieldChangeMeta.setColumnLabel(fieldName);
                    fieldChangeMeta.setChangeId(changeId);
                    Object oV = field.get(beanData);
                    if (oV instanceof String) {
                        String value = (String) field.get(beanData);
                        fieldChangeMeta.setColumnNewValue(value);
                        fieldChangeMeta.setColumnOldValue(value);
                    } else if (oV instanceof Number) {
                        String value = String.valueOf(field.get(beanData));
                        fieldChangeMeta.setColumnNewValue(value);
                        fieldChangeMeta.setColumnOldValue(value);
                    } else if (oV instanceof Date) {
                        String value = DateUtil.format((Date)oV, displayDefine.format());
                        fieldChangeMeta.setColumnNewValue(value);
                        fieldChangeMeta.setColumnOldValue(value);
                    }
                    changeData.add(fieldChangeMeta);
                }
            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
        }
        return changeData;
    }

    public List<FieldChangeMeta> buildDesignInfoNew(Object changeDataDto, String record) {
        return convertBeanField2Column2(changeDataDto, record);
    }

    private List<FieldChangeMeta> buildFactoryInfoNew(EquipFactoryChangeDataDto changeDataDto, String record) {
        return convertBeanField2Column2(changeDataDto, record);
    }

    private List<FieldChangeMeta> buildRegisterInfoNew(EquipRegisterChangeDataDto changeDataDto, String record) {
        return convertBeanField2Column2(changeDataDto, record);
    }

    public List<FieldChangeMeta> buildDesignInfoOld(String record) {
        EquipDesignChangeDataDto changeDataDto = new EquipDesignChangeDataDto();
        IdxBizJgDesignInfo designInfo = idxBizJgDesignInfoMapper.selectOne(new LambdaQueryWrapper<IdxBizJgDesignInfo>().eq(IdxBizJgDesignInfo::getRecord, record));
        BeanUtil.copyProperties(designInfo, changeDataDto);
        changeDataDto.setDesignDate(designInfo.getDesignDate());
        return convertBeanField2Column2(changeDataDto, record);
    }

    private List<FieldChangeMeta> buildFactoryInfoOld(String record) {
        EquipFactoryChangeDataDto changeDataDto = new EquipFactoryChangeDataDto();
        IdxBizJgFactoryInfo factoryInfo = jgFactoryInfoMapper.selectOne(new LambdaQueryWrapper<IdxBizJgFactoryInfo>().eq(IdxBizJgFactoryInfo::getRecord, record));
        BeanUtil.copyProperties(factoryInfo, changeDataDto);
        changeDataDto.setProduceDate(DateUtil.formatDate(factoryInfo.getProduceDate()));
        return convertBeanField2Column2(changeDataDto, record);
    }

    private List<FieldChangeMeta> buildRegisterInfoOld(String record) {
        EquipRegisterChangeDataDto changeDataDto = new EquipRegisterChangeDataDto();
        IdxBizJgRegisterInfo registerInfo = idxBizJgRegisterInfoMapper.selectOne(new LambdaQueryWrapper<IdxBizJgRegisterInfo>().eq(IdxBizJgRegisterInfo::getRecord, record));
        BeanUtil.copyProperties(registerInfo, changeDataDto);
        return convertBeanField2Column2(changeDataDto, record);
    }

    private List<FieldChangeMeta> buildUseInfoOld(String record) {
        EquipUseInfoChangeDataDto changeDataDto = new EquipUseInfoChangeDataDto();
        IdxBizJgUseInfo useInfo = idxBizJgUseInfoService.getOneData(record);
        BeanUtil.copyProperties(useInfo, changeDataDto);
        return convertBeanField2Column2(changeDataDto, record);
    }


    public Map<String, Object> cast2UnderCase(Map<String, Object> re) {
        // 由于历史遗留问题，和前端保存统一，要转成大写下滑线驼峰
        Map<String, Object> result = new HashMap<>();
        re.forEach((k, v) -> {
            result.put(CharSequenceUtil.toUnderlineCase(k).toUpperCase(), v);
        });
        return result;
    }

    public void castJsonFields(Map<String, Object> re) {
        jsonFields.forEach(field -> {
            if (re.containsKey(field) && re.get(field) instanceof String) {
                re.put(field, JSON.parse((String) re.get(field)));
            }
        });
    }

    public EquipDesignChangeDataDto buildDesignInfo(String record) {
        EquipDesignChangeDataDto changeDataDto = new EquipDesignChangeDataDto();
        IdxBizJgDesignInfo designInfo = idxBizJgDesignInfoMapper.selectOne(new LambdaQueryWrapper<IdxBizJgDesignInfo>().eq(IdxBizJgDesignInfo::getRecord, record));
        BeanUtil.copyProperties(designInfo, changeDataDto);
        changeDataDto.setDesignDate(designInfo.getDesignDate());
        return changeDataDto;
    }

    public EquipFactoryChangeDataDto buildFactoryInfo(String record) {
        EquipFactoryChangeDataDto changeDataDto = new EquipFactoryChangeDataDto();
        IdxBizJgFactoryInfo factoryInfo = jgFactoryInfoMapper.selectOne(new LambdaQueryWrapper<IdxBizJgFactoryInfo>().eq(IdxBizJgFactoryInfo::getRecord, record));
        BeanUtil.copyProperties(factoryInfo, changeDataDto);
        changeDataDto.setProduceDate(DateUtil.formatDate(factoryInfo.getProduceDate()));
        return changeDataDto;
    }

    public EquipRegisterChangeDataDto buildRegisterInfo(String record) {
        EquipRegisterChangeDataDto changeDataDto = new EquipRegisterChangeDataDto();
        IdxBizJgRegisterInfo registerInfo = idxBizJgRegisterInfoMapper.selectOne(new LambdaQueryWrapper<IdxBizJgRegisterInfo>().eq(IdxBizJgRegisterInfo::getRecord, record));
        BeanUtil.copyProperties(registerInfo, changeDataDto);
        return changeDataDto;
    }

    public EquipUseInfoChangeDataDto buildUseInfo(String record) {
        EquipUseInfoChangeDataDto changeDataDto = new EquipUseInfoChangeDataDto();
        IdxBizJgUseInfo useInfo = idxBizJgUseInfoService.getOneData(record);
        BeanUtil.copyProperties(useInfo, changeDataDto);
        return changeDataDto;
    }


    public void buildInspectChangeFields(String record, EquipInspectChangeDataDto inspectChangeDataDto, List<FieldChangeMeta> allChangeColumns) {
        if (StringUtils.isNotEmpty(inspectChangeDataDto.getJySeq())) {
            Map<String, Object> oldData = jgUseRegistrationService.getInspectDetail(inspectChangeDataDto.getJySeq());
            EquipInspectChangeDataDto inspectChangeDataOld = new EquipInspectChangeDataDto();
            BeanUtil.copyProperties(oldData, inspectChangeDataOld);
            List<FieldChangeMeta> newChangeFields = convertBeanField2Column2(inspectChangeDataDto, record);
            List<FieldChangeMeta> oldChangeFields = convertBeanField2Column2(inspectChangeDataOld, record);
            allChangeColumns.addAll(mergeChangeFields(newChangeFields, oldChangeFields));
        } else {
            List<FieldChangeMeta> newChangeFields = convertBeanField2Column2(inspectChangeDataDto, record);
            allChangeColumns.addAll(newChangeFields);
        }
    }




    private List<FieldChangeMeta> buildOtherInfoOld(String record) {
        EquipOtherInfoChangeDataDto changeDataDto = new EquipOtherInfoChangeDataDto();
        IdxBizJgOtherInfo otherInfo = idxBizJgOtherInfoMapper.selectOne(new LambdaQueryWrapper<IdxBizJgOtherInfo>().eq(IdxBizJgOtherInfo::getRecord, record));
        BeanUtil.copyProperties(otherInfo, changeDataDto);
        return convertBeanField2Column2(changeDataDto, record);
    }

    /**
     * 简化版字段对比更新工具
     * 要求：对象字段名与数据库列名一致（驼峰转下划线自动处理）
     *
     * @param mapper MyBatis-Plus 的 Mapper 实例（直接传入）
     * @param oldObj 旧对象（必须有id字段）
     * @param newObj 新对象（必须与旧对象同一类型）
     */
    public <T> List<FieldChangeMeta> simpleTrackAndUpdate(BaseMapper<T> mapper, T oldObj, T newObj, String changeId, String whereKey, String whereValue) {
        List<FieldChangeMeta> changeData = new ArrayList<>();
        // 基础校验
        if (oldObj == null || newObj == null) {
            throw new IllegalArgumentException("新旧对象不能为空");
        }
        if (!oldObj.getClass().equals(newObj.getClass())) {
            throw new IllegalArgumentException("对象类型不一致");
        }
        Group group = newObj.getClass().getAnnotation(Group.class);
        Class<?> clazz = oldObj.getClass();
        UpdateWrapper<T> wrapper = new UpdateWrapper<>();
        // 遍历所有字段（包含父类）
        for (Field field : getAllFields(clazz)) {
            try {
                field.setAccessible(true);
                Object oldVal = field.get(oldObj);
                Object newVal = field.get(newObj);
                FieldDisplayDefine displayDefine = field.getAnnotation(FieldDisplayDefine.class);
                TableField tableField = field.getAnnotation(TableField.class);
                // 对比逻辑
                if (!Objects.equals(oldVal, newVal)) {
                    if (displayDefine != null && displayDefine.isExist()) {
                        String columnName = tableField.value();
                        wrapper.set(columnName, newVal);
                        String fieldName = displayDefine.value();
                        FieldChangeMeta fieldChangeMeta = new FieldChangeMeta();
                        fieldChangeMeta.setColumnKey(field.getName());
                        fieldChangeMeta.setColumnFamily(group.value());
                        fieldChangeMeta.setColumnLabel(fieldName);
                        fieldChangeMeta.setChangeId(changeId);
                        if (newVal instanceof String) {
                            fieldChangeMeta.setColumnOldValue((String) oldVal);
                            fieldChangeMeta.setColumnNewValue((String) newVal);
                        } else if (newVal instanceof Number) {
                            fieldChangeMeta.setColumnOldValue(String.valueOf(oldVal));
                            fieldChangeMeta.setColumnNewValue(String.valueOf(newVal));
                        } else if (newVal instanceof Date) {
                            fieldChangeMeta.setColumnOldValue(DateUtil.formatDate((Date) oldVal));
                            fieldChangeMeta.setColumnNewValue(DateUtil.formatDate((Date) newVal));
                        } else if (newVal instanceof Boolean) {
                            fieldChangeMeta.setColumnOldValue(Boolean.toString((Boolean) oldVal));
                            fieldChangeMeta.setColumnNewValue(Boolean.toString((Boolean) newVal));
                        }
                        changeData.add(fieldChangeMeta);
                    }
                }

            } catch (IllegalAccessException e) {
                System.err.println("字段访问失败: " + field.getName());
            }
        }
        // 执行更新
        if (!changeData.isEmpty()) {
            wrapper.eq(whereKey, whereValue);
            mapper.update(null, wrapper);
        }
        return changeData;
    }
}
